# -*- coding: utf-8 -*-

from ....Methods import ParentMissingError


def is_outwards(self):
    """Return if the magnet is outwards (on an external lamination) or inwards
    (on an internal lamination)

    Parameters
    ----------
    self : Magnet
        A Magnet object

    Returns
    -------
    is_outwards: bool
        Direction of the magnet
    Raises
    _______
    ParentMissingError
        Error: The magnet is not inside a Slot
    """
    if self.parent is not None:
        return self.parent.is_outwards()
    else:
        raise ParentMissingError("Error: The magnet is not inside a Slot")



# -*- coding: utf-8 -*-

from numpy import exp

from ....Classes.Arc2 import Arc2
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Segment) needed to plot the object.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW22
        A SlotW22 object

    Returns
    -------
    curve_list: llist
        A list of 5 Segment and 3 Arc2

    """
    Rbo = self.get_Rbo()

    # comp point coordinate (in complex)
    Z1 = Rbo * exp(-1j * self.W0 / 2)
    if self.is_outwards():
        R1 = Rbo + self.H0
        R2 = Rbo + self.H0 + self.H2
    else:  # inward slot
        R1 = Rbo - self.H0
        R2 = Rbo - self.H0 - self.H2
    Z2 = R1 * exp(-1j * self.W0 / 2.0)
    Z3 = R1 * exp(-1j * self.W2 / 2.0)
    Z4 = R2 * exp(-1j * self.W2 / 2.0)

    # symetry
    Z5 = Z4.conjugate()
    Z6 = Z3.conjugate()
    Z7 = Z2.conjugate()
    Z8 = Z1.conjugate()
    Zc = 0

    # Creation of curve
    curve_list = list()
    if self.H0 > 0:
        curve_list.append(Segment(Z1, Z2))
    if self.W2 != self.W0:
        curve_list.append(Arc2(Z2, Zc, -(self.W2 - self.W0) / 2))
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Arc2(Z4, Zc, self.W2))
    curve_list.append(Segment(Z5, Z6))
    if self.W2 != self.W0:
        curve_list.append(Arc2(Z6, Zc, -(self.W2 - self.W0) / 2))
    if self.H0 > 0:
        curve_list.append(Segment(Z7, Z8))

    return curve_list



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve
    in the list

    Parameters
    ----------
    self : SlotUD
        A SlotUD object

    Returns
    -------
    curve_list: list
        A list of Segments

    """

    # Apply symmetry if needed
    point_list = self.point_list.copy()
    if self.is_sym:
        for point in self.point_list[::-1]:
            point_list.append(point.conjugate())

    # Creation of curve
    curve_list = list()
    for ii in range(len(point_list) - 1):
        curve_list.append(Segment(point_list[ii], point_list[ii + 1]))

    return curve_list



# -*- coding: utf-8 -*-

import numpy as np


def grad_shape_function(self, point):
    """ Return the gradient of linear shape functions in reference triangle for a given point """

    values = np.zeros([2, 3], dtype=float)
    [x, y] = point[0:2]

    if (x >= 0) or (y >= 0) or (1 - x - y >= 0):
        values[0, 0] = -1
        values[0, 1] = 1
        values[0, 2] = 0
        values[1, 0] = -1
        values[1, 1] = 0
        values[1, 2] = 1

    return values



# -*- coding: utf-8 -*-

from numpy import arcsin, exp

from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW27
        A SlotW27 object

    Returns
    -------
    curve_list: list
        A list of 9 Segment

    """
    Rbo = self.get_Rbo()

    # alpha is the angle to rotate Z0 so ||Z1,Z10|| = W0
    alpha = float(arcsin(self.W0 / (2 * Rbo)))

    # comp point coordinate (in complex)
    Z0 = Rbo * exp(1j * 0)
    Z1 = Z0 * exp(1j * alpha)

    if self.is_outwards():
        Z2 = Z1 + self.H0
        Z3 = Z2 + (self.W1 - self.W0) * 1j / 2.0
        Z4 = Z3 + self.H1 + (self.W2 - self.W1) / 2.0 * 1j
        Z5 = Z4 + self.H2 + (self.W3 - self.W2) / 2.0 * 1j
    else:  # inward slot
        Z2 = Z1 - self.H0
        Z3 = Z2 + (self.W1 - self.W0) * 1j / 2.0
        Z4 = Z3 - self.H1 + (self.W2 - self.W1) / 2.0 * 1j
        Z5 = Z4 - self.H2 + (self.W3 - self.W2) / 2.0 * 1j

    # symetry
    Z6 = Z5.conjugate()
    Z7 = Z4.conjugate()
    Z8 = Z3.conjugate()
    Z9 = Z2.conjugate()
    Z10 = Z1.conjugate()
    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10] = [
        Z10,
        Z9,
        Z8,
        Z7,
        Z6,
        Z5,
        Z4,
        Z3,
        Z2,
        Z1,
    ]

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Segment(Z4, Z5))
    curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Segment(Z9, Z10))

    return curve_list



# -*- coding: utf-8 -*-


def comp_radius_mid_wind(self):
    """Compute the radius at the middle of the winding part of the slot

    Parameters
    ----------
    self : SlotWind
        A SlotWind object

    Returns
    -------
    Rmw: float
        Mid winding radius [m]

    """

    Rbo = self.get_Rbo()
    Hslot = self.comp_height()
    Hwind = self.comp_height_wind()
    if self.is_outwards():
        return Rbo + Hslot - Hwind / 2
    else:
        return Rbo - Hslot + Hwind / 2



# -*- coding: utf-8 -*-
from numpy import angle, pi, floor_divide
from ...Classes.HoleM50 import HoleM50
from ...Classes.HoleM51 import HoleM51
from ...Classes.HoleM52 import HoleM52
from ...Classes.HoleM53 import HoleM53
from ...Methods import NotImplementedYetError
from ...Functions.FEMM.get_mesh_param import get_mesh_param


def assign_FEMM_surface(femm, surf, prop, FEMM_dict, rotor, stator):
    """Assign the property given in parameter to surface having the label given
    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    surf : Surface
        the surface to assign
    prop : str
        The property to assign in FEMM
    FEMM_dict : dict
        Dictionnary containing the main parameters of FEMM
    rotor : Lamination
        The rotor of the machine
    stator : Lamination
        The stator of the machine
    Returns
    -------
    None

    """

    mesh_dict = get_mesh_param(surf.label, FEMM_dict)
    label = surf.label
    Clabel = 0  # By default no circuit
    Ntcoil = 0  # By default no circuit
    mag = 0  # By default no magnetization

    # Select the lamination according to the label
    if "Rotor" in label:
        lam = rotor
    else:
        lam = stator

    # point_ref is None => don't assign the surface
    if surf.point_ref is not None:
        # Select the surface
        point_ref = surf.point_ref
        femm.mi_addblocklabel(point_ref.real, point_ref.imag)
        femm.mi_selectlabel(point_ref.real, point_ref.imag)

        # Get circuit or magnetization properties if needed
        if "Wind" in label:  # If the surface is a winding
            if "Rotor" in label:  # Winding on the rotor
                Clabel = "Circr" + prop[2]
            else:  # winding on the stator
                Clabel = "Circs" + prop[2]
            Ntcoil = lam.winding.Ntcoil
            if prop[-1] == "-":  # Adapt Ntcoil sign if needed
                Ntcoil *= -1
        elif "HoleMagnet" in label:  # LamHole
            if "Parallel" in label:
                # calculate pole angle and angle of pole middle
                alpha_p = 360 / lam.hole[0].Zh
                mag_0 = (
                    floor_divide(angle(point_ref, deg=True), alpha_p) + 0.5
                ) * alpha_p

                # HoleM50 or HoleM53
                if (type(lam.hole[0]) == HoleM50) or (type(lam.hole[0]) == HoleM53):
                    if "_T0_" in label:
                        mag = mag_0 + lam.hole[0].comp_alpha() * 180 / pi
                    else:
                        mag = mag_0 - lam.hole[0].comp_alpha() * 180 / pi

                # HoleM51
                if type(lam.hole[0]) == HoleM51:
                    if "_T0_" in label:
                        mag = mag_0 + lam.hole[0].comp_alpha() * 180 / pi
                    elif "_T1_" in label:
                        mag = mag_0
                    else:
                        mag = mag_0 - lam.hole[0].comp_alpha() * 180 / pi

                # HoleM52
                if type(lam.hole[0]) == HoleM52:
                    mag = mag_0

                # modifiy magnetisation of south poles
                if "_S_" in label:
                    mag = mag + 180
            else:
                raise NotImplementedYetError(
                    "Only parallele magnetization are available for HoleMagnet"
                )
        elif "Magnet" in label:  # LamSlotMag
            if "Radial" in label and "_N_" in label:  # Radial magnetization
                mag = "theta"  # North pole magnet
            elif "Radial" in label:
                mag = "theta + 180"  # South pole magnet
            elif "Parallel" in label and "_N_" in label:
                mag = angle(point_ref) * 180 / pi  # North pole magnet
            elif "Parallel" in label:
                mag = angle(point_ref) * 180 / pi + 180  # South pole magnet
            elif "Hallbach" in label:
                Zs = lam.slot.Zs
                mag = str(-(Zs / 2 - 1)) + " * theta + 90 "
        elif "Ventilation" in label:
            prop = "Air"
        elif "Hole" in label:
            prop = "Air"
        # Set the surface property
        femm.mi_setblockprop(
            prop,
            mesh_dict["automesh"],
            mesh_dict["meshsize"],
            Clabel,
            mag,
            mesh_dict["group"],
            Ntcoil,
        )
        femm.mi_clearselected()



# -*- coding: utf-8 -*-
from PySide2.QtCore import Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QDialog, QMessageBox

from ......Classes.MagnetType13 import MagnetType13
from ......Classes.SlotMFlat import SlotMFlat
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SMagnet.PMagnet13.Gen_PMagnet13 import Gen_PMagnet13


class PMagnet13(Gen_PMagnet13, QDialog):
    """Page to set the Magnet Type 13"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for Magnet combobox
    mag_name = "Flat bottom, curved top"
    mag_type = MagnetType13
    slot_type = SlotMFlat

    def __init__(self, machine=None):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 widget
        machine : Machine
            current machine to edit
        """
        # Build the interface according to the .ui file
        QDialog.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine

        # Set FloatEdit unit
        self.lf_H0.unit = "m"
        self.lf_Hmag.unit = "m"
        self.lf_Wmag.unit = "m"
        self.lf_Rtopm.unit = "m"
        self.u = gui_option.unit
        # Set unit name (m ou mm)
        wid_list = [self.unit_Rtopm, self.unit_H0, self.unit_Hmag, self.unit_Wmag]
        for wid in wid_list:
            wid.setText(self.u.get_m_name())

        # Fill the fields with the lamination values (if they're filled)
        self.lf_Hmag.setValue(self.machine.rotor.slot.magnet[0].Hmag)
        self.lf_Wmag.setValue(self.machine.rotor.slot.magnet[0].Wmag)
        self.lf_Rtopm.setValue(self.machine.rotor.slot.magnet[0].Rtop)
        if self.machine.type_machine == 6 and self.machine.rotor.slot.H0 is None:
            self.machine.rotor.slot.H0 = 0  # Default value for SPMSM
        self.lf_H0.setValue(self.machine.rotor.slot.H0)

        # Display the main output of the slot (surface, height...)
        self.w_out.comp_output()

        # Connect the signal/slot
        self.lf_Hmag.editingFinished.connect(self.set_Hmag)
        self.lf_Wmag.editingFinished.connect(self.set_Wmag)
        self.lf_Rtopm.editingFinished.connect(self.set_Rtopm)
        self.lf_H0.editingFinished.connect(self.set_H0)

    def set_H0(self):
        """Signal to update the value of H0 according to the line edit

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 object
        """
        self.machine.rotor.slot.H0 = self.lf_H0.value()
        if self.machine.rotor.slot.H0 > 0:
            self.machine.type_machine = 7  # Inset
        else:
            self.machine.type_machine = 6  # Surface
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Wmag(self):
        """Signal to update the value of Wmag according to the line edit

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 object
        """
        self.machine.rotor.slot.magnet[0].Wmag = self.lf_Wmag.value()
        self.machine.rotor.slot.W0 = self.lf_Wmag.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Rtopm(self):
        """Signal to update the value of Rtopm according to the line edit

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 object
        """
        self.machine.rotor.slot.magnet[0].Rtop = self.lf_Rtopm.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Hmag(self):
        """Signal to update the value of Hmag according to the line edit

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 object
        """
        self.machine.rotor.slot.magnet[0].Hmag = self.lf_Hmag.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def check(self):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 object

        Returns
        -------
        error: str
            Error message (return None if no error)
        """
        # Check that everything is set
        if self.machine.rotor.slot.magnet[0].Wmag is None:
            return self.tr("You must set Wmag !")
        if self.machine.rotor.slot.magnet[0].Hmag is None:
            return self.tr("You must set Hmag !")
        if self.machine.rotor.slot.H0 is None:
            return self.tr("You must set H0 !")

        # Check that everything is set right
        try:
            mec_gap = self.machine.comp_width_airgap_mec()
        except:
            return self.tr("Unable to draw the magnet, " "please check your geometry !")

        if mec_gap <= 0:
            return self.tr("You must have gap_min > 0 (reduce Hmag) !")
        if (
            self.machine.rotor.slot.magnet[0].Rtop
            < self.machine.rotor.slot.magnet[0].Wmag / 2.0
        ):
            return self.tr("You must have Rtopm >= Wmag/2 !")

    def comp_output(self):
        """Compute and display the magnet output

        Parameters
        ----------
        self : PMagnet13
            A PMagnet13 object
        """
        # Gap is set in SMachineDimension
        gap = format(self.u.get_m(self.machine.comp_width_airgap_mag()), ".4g")
        self.out_gap.setText(self.tr("gap: ") + gap + " " + self.u.get_m_name())

        mag_txt = self.tr("Magnet surface: ")
        gm_txt = self.tr("gap_min: ")
        taum_txt = self.tr("taum: ")

        if self.check() is None:
            # We compute the output only if the slot is correctly set
            Zs = self.machine.rotor.slot.Zs
            # Compute all the needed output as string (scientific notation with
            # 2 digits)
            Smag = format(
                self.u.get_m2(self.machine.rotor.slot.magnet[0].comp_surface()), ".4g"
            )
            gap_min = format(self.u.get_m(self.machine.comp_width_airgap_mec()), ".4g")

            taum = 100 * self.machine.rotor.slot.magnet[0].comp_ratio_opening(Zs / 2.0)
            taum = "%.4g" % (taum)

            # Update the GUI to display the Output
            self.out_Smag.setText(mag_txt + Smag + " " + self.u.get_m2_name())
            self.out_gap_min.setText(gm_txt + gap_min + " " + self.u.get_m_name())
            self.out_taum.setText(taum_txt + taum + " %")
        else:
            # We can't compute the output => We erase the previous version
            # (that way the user know that something is wrong without
            # clicking next)
            self.out_Smag.setText(mag_txt + "?")
            self.out_gap_min.setText(gm_txt + "?")
            self.out_taum.setText(taum_txt + " ?")



# -*- coding: utf-8 -*-

from ....Functions.FEMM.draw_FEMM import draw_FEMM
from ....Classes._FEMMHandler import FEMMHandler


def comp_flux_airgap(self, output):
    """Compute using FEMM the flux in the airgap

    Parameters
    ----------
    self : MagFEMM
        a MagFEMM object
    output : Output
        an Output object
    """

    # Set the symmetry factor according to the machine
    if self.is_periodicity_a:
        (sym, is_antiper_a, _, _) = self.parent.parent.get_machine_periodicity()
        if is_antiper_a:
            sym = sym * 2
    else:
        sym = 1
        is_antiper_a = False

    # Setup the FEMM simulation
    # Geometry building and assigning property in FEMM
    # Instanciate a new FEMM
    femm = FEMMHandler(not self.is_close_femm)
    if not self.import_file:  # True if None or len == 0
        self.get_logger().debug("Drawing machine in FEMM...")
        output.mag.FEMM_dict = draw_FEMM(
            femm,
            output,
            is_mmfr=self.is_mmfr,
            is_mmfs=self.is_mmfs,
            sym=sym,
            is_antiper=is_antiper_a,
            type_calc_leakage=self.type_calc_leakage,
            is_remove_vent=self.is_remove_vent,
            is_remove_slotS=self.is_remove_slotS,
            is_remove_slotR=self.is_remove_slotR,
            type_BH_stator=self.type_BH_stator,
            type_BH_rotor=self.type_BH_rotor,
            kgeo_fineness=self.Kgeo_fineness,
            kmesh_fineness=self.Kmesh_fineness,
            user_FEMM_dict=self.FEMM_dict,
            path_save=self.get_path_save_fem(output),
            is_sliding_band=self.is_sliding_band,
            transform_list=self.transform_list,
            rotor_dxf=self.rotor_dxf,
            stator_dxf=self.stator_dxf,
        )
    else:
        self.get_logger().debug("Reusing the FEMM file: " + self.import_file)
        output.mag.FEMM_dict = self.FEMM_dict
        # Open the document
        femm.openfemm()
        femm.main_minimize()
        femm.opendocument(self.import_file)

    # Solve for all time step and store all the results in output
    self.solve_FEMM(femm, output, sym)



# -*- coding: utf-8 -*-
from pandas import read_excel
from ....Methods.Import.ImportMatrixXls import XlsFileError
from os.path import isfile

from ....Functions.path_tools import abs_file_path


def get_data(self):
    """Return the object's matrix

    Parameters
    ----------
    self : ImportMatrixVal
        An ImportMatrixVal object

    Returns
    -------
    matrix: ndarray
        The object's matrix

    """
    file_path = abs_file_path(self.file_path, is_check=False)
    if not isfile(file_path):
        raise XlsFileError("ERROR: The xls file doesn't exist " + self.file_path)
    if self.usecols == "":
        self.usecols = None
    df = read_excel(
        file_path, self.sheet, header=None, usecols=self.usecols, skiprows=self.skiprows
    )
    return self.edit_matrix(df.values)



# -*- coding: utf-8 -*-


def comp_surface(self):
    """Compute the surface of the conductor

    Parameters
    ----------
    self : CondType11
        A CondType11 object

    Returns
    -------
    S: float
        Surface of the conductor (with insulation) [m**2]

    """

    S = self.comp_height() * self.comp_width()

    return S



from ....Classes.DataKeeper import DataKeeper
from ....Classes.InputCurrent import InputCurrent


def gen_datakeeper_list(self):
    """Generate default DataKeepers for VarLoadSpeed workflow"""
    datakeeper_list = []

    simu = self.parent
    # Save speed
    datakeeper_list.append(
        DataKeeper(
            name="Speed",
            symbol="N0",
            unit="rpm",
            keeper="lambda output: output.elec.N0",
        )
    )

    # Get default datakeeper
    if simu and (simu.elec or isinstance(simu.input, InputCurrent)):
        datakeeper_list.extend(self.get_elec_datakeeper())
    if simu and simu.mag:
        datakeeper_list.extend(self.get_mag_datakeeper())
    if simu and simu.force:
        datakeeper_list.extend(self.get_force_datakeeper())

    self.datakeeper_list.extend(datakeeper_list)



# -*- coding: utf-8 -*-


def comp_surface_wind(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW13
        A SlotW13 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    return 0.5 * (self.W2 + self.W3) * self.H2



# -*- coding: utf-8 -*-

from numpy import arcsin, exp, angle

from ....Classes.Segment import Segment

from ....Classes.Arc1 import Arc1


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW23
        A SlotW23 object

    Returns
    -------
    curve_list: list
        A list of 6 Segment and 1 Arc

    """

    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8] = self._comp_point_coordinate()

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Arc1(Z4, Z5, abs(Z5)))
    curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z8))

    return curve_list



# -*- coding: utf-8 -*-

from numpy import angle
from scipy.optimize import fsolve

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine


def build_geometry_wind(self, Nrad, Ntan, is_simplified=False, alpha=0, delta=0):
    """Split the slot winding area in several zone

    Parameters
    ----------
    self : SlotW11
        A SlotW11 object
    Nrad : int
        Number of radial layer
    Ntan : int
        Number of tangentiel layer
    is_simplified : bool
        boolean to specify if coincident lines are considered as one or different lines (Default value = False)
    alpha : float
        number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_list: list
        List of surface delimiting the winding zone

    """

    if self.get_is_stator():  # check if the slot is on the stator
        st = "S"
    else:
        st = "R"

    [Z10, Z9, Z8, Z7, Z6, Z5, Z4, Z3, Z2, Z1, rot_sign] = self._comp_point_coordinate()
    rot_sign = -rot_sign
    Ztan1 = (Z3 + Z8) / 2.0
    Ztan2 = (Z5 + Z6) / 2.0

    if self.H2 / 2.0 > self.R1:  # Zrad1 between Z8 and Z7
        Zmid = (Ztan1 + Ztan2) / 2.0
        x = fsolve(
            lambda x: angle((Z7 - (Zmid + 1j * x)) / (Z7 - Z8)),
            -(self.W2 + self.W1) / 4.0,
        )
        Zrad1 = Zmid + 1j * x[0]
        Zrad2 = Zrad1.conjugate()

    # Alternative points for uneven winding layers (cf ICEM 2020 publication)
    # Zmid1 = (Ztan1 + Ztan2) / 2
    # Zmid1 = (Zmid1 + Ztan1) / 2
    # Zmid2 = (Ztan1 + Ztan2) / 2
    # Zmid2 = (Zmid2 + Ztan2) / 2

    # x = fsolve(
    #     lambda x: angle((Z7 - (Zmid1 + 1j * x)) / (Z7 - Z8)), -(self.W2 + self.W1) / 4.0
    # )
    # Zrad1 = Zmid1 + 1j * x[0]

    # x = fsolve(
    #     lambda x: angle((Z4 - (Zmid2 + 1j * x)) / (Z4 - Z3)), (self.W2 + self.W1) / 4.0
    # )
    # Zrad2 = Zmid2 + 1j * x[0]
    # is_uneven = True

    surf_list = list()
    if Nrad == 1 and Ntan == 2:
        if is_simplified:  # no coincident lines allowed
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z8, Ztan1))
            curve_list.append(Segment(Ztan1, Ztan2))
            point_ref = (Z8 + Ztan1 + Ztan2 + Z6 + Z7) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z3))
            point_ref = (Ztan1 + Z3 + Z4 + Z5 + Ztan2) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
        else:
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z8, Ztan1))
            curve_list.append(Segment(Ztan1, Ztan2))
            if self.W2 > 2 * self.R1:
                curve_list.append(Segment(Ztan2, Z6))
            curve_list.append(
                Arc1(Z6, Z7, rot_sign * self.R1, is_trigo_direction=False)
            )
            curve_list.append(Segment(Z7, Z8))
            point_ref = (Z8 + Ztan1 + Ztan2 + Z6 + Z7) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z3))
            curve_list.append(Segment(Z3, Z4))
            curve_list.append(
                Arc1(Z4, Z5, rot_sign * self.R1, is_trigo_direction=False)
            )
            if self.W2 > 2 * self.R1:
                curve_list.append(Segment(Z5, Ztan2))
            curve_list.append(Segment(Ztan2, Ztan1))
            point_ref = (Ztan1 + Z3 + Z4 + Z5 + Ztan2) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
    # elif is_uneven:  # For ICEM 2020 publication
    #     # Part 1 (0,0)
    #     curve_list = list()
    #     curve_list.append(Segment(Z8, Ztan1))
    #     curve_list.append(Segment(Ztan1, Zmid1))
    #     curve_list.append(Segment(Zmid1, Zrad1))
    #     curve_list.append(Segment(Zrad1, Z8))
    #     point_ref = (Z8 + Ztan1 + Zmid1 + Zrad1) / 4
    #     surface = SurfLine(
    #         line_list=curve_list, label="Wind" + st + "_R0_T0_S0", point_ref=point_ref
    #     )
    #     surf_list.append(surface)
    #     # Part2 (1,0)
    #     curve_list = list()
    #     curve_list.append(Segment(Zrad1, Zmid1))
    #     curve_list.append(Segment(Zmid1, Ztan2))
    #     if self.W2 > 2 * self.R1:
    #         curve_list.append(Segment(Ztan2, Z6))
    #     curve_list.append(Arc1(Z6, Z7, rot_sign * self.R1, is_trigo_direction=True))
    #     curve_list.append(Segment(Z7, Zrad1))
    #     point_ref = (Zrad1 + Zmid1 + Ztan2 + Z6 + Z7) / 5
    #     surface = SurfLine(
    #         line_list=curve_list, label="Wind" + st + "_R1_T0_S0", point_ref=point_ref
    #     )
    #     surf_list.append(surface)
    #     # Part3 (0,1)
    #     curve_list = list()
    #     curve_list.append(Segment(Ztan1, Z3))
    #     curve_list.append(Segment(Z3, Zrad2))
    #     curve_list.append(Segment(Zrad2, Zmid2))
    #     curve_list.append(Segment(Zmid2, Ztan1))
    #     point_ref = (Ztan1 + Z3 + Zrad2 + Zmid2) / 4
    #     surface = SurfLine(
    #         line_list=curve_list, label="Wind" + st + "_R0_T1_S0", point_ref=point_ref
    #     )
    #     surf_list.append(surface)
    #     # Part4 (1,1)
    #     curve_list = list()
    #     curve_list.append(Segment(Zmid2, Zrad2))
    #     curve_list.append(Segment(Zrad2, Z4))
    #     curve_list.append(Arc1(Z4, Z5, rot_sign * self.R1, is_trigo_direction=True))
    #     if self.W2 > 2 * self.R1:
    #         curve_list.append(Segment(Z5, Ztan2))
    #     curve_list.append(Segment(Ztan2, Zmid2))
    #     point_ref = (Zmid2 + Zrad2 + Z4 + Z5 + Ztan2) / 5
    #     surface = SurfLine(
    #         line_list=curve_list, label="Wind" + st + "_R1_T1_S0", point_ref=point_ref
    #     )
    #     surf_list.append(surface)
    elif Nrad == 2 and Ntan == 1 and self.H2 / 2.0 > self.R1:
        if is_simplified:  # no coincident lines allowed
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z8, Z3))
            curve_list.append(Segment(Zrad2, Zrad1))
            point_ref = (Z8 + Z3 + Zrad2 + Zrad1) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            point_ref = (Zrad2 + Zrad1 + Z4 + Z5 + Z6 + Z7) / 6
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
        else:
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z8, Z3))
            curve_list.append(Segment(Z3, Zrad2))
            curve_list.append(Segment(Zrad2, Zrad1))
            curve_list.append(Segment(Zrad1, Z8))
            point_ref = (Z8 + Z3 + Zrad2 + Zrad1) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            curve_list.append(Segment(Zrad1, Zrad2))
            curve_list.append(Segment(Zrad2, Z4))
            curve_list.append(
                Arc1(Z4, Z5, rot_sign * self.R1, is_trigo_direction=False)
            )
            if self.W2 > 2 * self.R1:
                curve_list.append(Segment(Z5, Z6))
            curve_list.append(
                Arc1(Z6, Z7, rot_sign * self.R1, is_trigo_direction=False)
            )
            curve_list.append(Segment(Z7, Zrad1))
            point_ref = (Zrad2 + Zrad1 + Z4 + Z5 + Z6 + Z7) / 6
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
    elif Nrad == 2 and Ntan == 2 and self.H2 / 2.0 > self.R1:
        if is_simplified:  # no coincident lines allowed
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z8, Ztan1))
            curve_list.append(Segment(Ztan1, Zmid))
            curve_list.append(Segment(Zmid, Zrad1))
            point_ref = (Z8 + Ztan1 + Zmid + Zrad1) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            curve_list.append(Segment(Zmid, Ztan2))
            point_ref = (Zrad1 + Zmid + Ztan2 + Z6 + Z7) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part3 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z3))
            curve_list.append(Segment(Zrad2, Zmid))
            point_ref = (Ztan1 + Z3 + Zrad2 + Zmid) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part4 (1,1)
            curve_list = list()
            point_ref = (Zmid + Zrad2 + Z4 + Z5 + Ztan2) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
        else:

            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z8, Ztan1))
            curve_list.append(Segment(Ztan1, Zmid))
            curve_list.append(Segment(Zmid, Zrad1))
            curve_list.append(Segment(Zrad1, Z8))
            point_ref = (Z8 + Ztan1 + Zmid + Zrad1) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            curve_list.append(Segment(Zrad1, Zmid))
            curve_list.append(Segment(Zmid, Ztan2))
            if self.W2 > 2 * self.R1:
                curve_list.append(Segment(Ztan2, Z6))
            curve_list.append(Arc1(Z6, Z7, rot_sign * self.R1, is_trigo_direction=True))
            curve_list.append(Segment(Z7, Zrad1))
            point_ref = (Zrad1 + Zmid + Ztan2 + Z6 + Z7) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part3 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z3))
            curve_list.append(Segment(Z3, Zrad2))
            curve_list.append(Segment(Zrad2, Zmid))
            curve_list.append(Segment(Zmid, Ztan1))
            point_ref = (Ztan1 + Z3 + Zrad2 + Zmid) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part4 (1,1)
            curve_list = list()
            curve_list.append(Segment(Zmid, Zrad2))
            curve_list.append(Segment(Zrad2, Z4))
            curve_list.append(Arc1(Z4, Z5, rot_sign * self.R1, is_trigo_direction=True))
            if self.W2 > 2 * self.R1:
                curve_list.append(Segment(Z5, Ztan2))
            curve_list.append(Segment(Ztan2, Zmid))
            point_ref = (Zmid + Zrad2 + Z4 + Z5 + Ztan2) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
    else:
        # Creation of curve
        curve_list = list()
        curve_list.append(Segment(Z8, Z3))
        curve_list.append(Segment(Z3, Z4))
        if self.R1 * 2 < self.W2:
            curve_list.append(
                Arc1(
                    Z4,
                    Z5,
                    rot_sign * self.R1,
                    is_trigo_direction=not self.is_outwards(),
                )
            )
            curve_list.append(Segment(Z5, Z6))
            curve_list.append(
                Arc1(
                    Z6,
                    Z7,
                    rot_sign * self.R1,
                    is_trigo_direction=not self.is_outwards(),
                )
            )
        else:  # Z5 == Z6
            curve_list.append(Arc3(Z4, Z7, not self.is_outwards()))
        curve_list.append(Segment(Z7, Z8))
        surface = SurfLine(
            line_list=curve_list, label="Wind" + st + "_R0_T0_S0", point_ref=Zmid
        )
        surf_list.append(surface)

    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)
    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/Simulation.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/Simulation
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.Simulation.run import run
except ImportError as error:
    run = error


from ._check import InitUnKnowClassError
from .Machine import Machine
from .Input import Input
from .VarSimu import VarSimu
from .Post import Post


class Simulation(FrozenClass):
    """Abstract class for the simulation"""

    VERSION = 1

    # cf Methods.Simulation.Simulation.run
    if isinstance(run, ImportError):
        run = property(
            fget=lambda x: raise_(
                ImportError("Can't use Simulation method run: " + str(run))
            )
        )
    else:
        run = run
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name="",
        desc="",
        machine=-1,
        input=-1,
        logger_name="Pyleecan.Simulation",
        var_simu=None,
        postproc_list=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "machine" in list(init_dict.keys()):
                machine = init_dict["machine"]
            if "input" in list(init_dict.keys()):
                input = init_dict["input"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "var_simu" in list(init_dict.keys()):
                var_simu = init_dict["var_simu"]
            if "postproc_list" in list(init_dict.keys()):
                postproc_list = init_dict["postproc_list"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.name = name
        self.desc = desc
        self.machine = machine
        self.input = input
        self.logger_name = logger_name
        self.var_simu = var_simu
        self.postproc_list = postproc_list

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Simulation_str = ""
        if self.parent is None:
            Simulation_str += "parent = None " + linesep
        else:
            Simulation_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Simulation_str += 'name = "' + str(self.name) + '"' + linesep
        Simulation_str += 'desc = "' + str(self.desc) + '"' + linesep
        if self.machine is not None:
            tmp = self.machine.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simulation_str += "machine = " + tmp
        else:
            Simulation_str += "machine = None" + linesep + linesep
        if self.input is not None:
            tmp = self.input.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simulation_str += "input = " + tmp
        else:
            Simulation_str += "input = None" + linesep + linesep
        Simulation_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        if self.var_simu is not None:
            tmp = self.var_simu.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simulation_str += "var_simu = " + tmp
        else:
            Simulation_str += "var_simu = None" + linesep + linesep
        if len(self.postproc_list) == 0:
            Simulation_str += "postproc_list = []" + linesep
        for ii in range(len(self.postproc_list)):
            tmp = (
                self.postproc_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            Simulation_str += (
                "postproc_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        return Simulation_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.name != self.name:
            return False
        if other.desc != self.desc:
            return False
        if other.machine != self.machine:
            return False
        if other.input != self.input:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.var_simu != self.var_simu:
            return False
        if other.postproc_list != self.postproc_list:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        Simulation_dict = dict()
        Simulation_dict["name"] = self.name
        Simulation_dict["desc"] = self.desc
        if self.machine is None:
            Simulation_dict["machine"] = None
        else:
            Simulation_dict["machine"] = self.machine.as_dict()
        if self.input is None:
            Simulation_dict["input"] = None
        else:
            Simulation_dict["input"] = self.input.as_dict()
        Simulation_dict["logger_name"] = self.logger_name
        if self.var_simu is None:
            Simulation_dict["var_simu"] = None
        else:
            Simulation_dict["var_simu"] = self.var_simu.as_dict()
        if self.postproc_list is None:
            Simulation_dict["postproc_list"] = None
        else:
            Simulation_dict["postproc_list"] = list()
            for obj in self.postproc_list:
                Simulation_dict["postproc_list"].append(obj.as_dict())
        # The class name is added to the dict for deserialisation purpose
        Simulation_dict["__class__"] = "Simulation"
        return Simulation_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.name = None
        self.desc = None
        if self.machine is not None:
            self.machine._set_None()
        if self.input is not None:
            self.input._set_None()
        self.logger_name = None
        if self.var_simu is not None:
            self.var_simu._set_None()
        for obj in self.postproc_list:
            obj._set_None()

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the simulation

        :Type: str
        """,
    )

    def _get_desc(self):
        """getter of desc"""
        return self._desc

    def _set_desc(self, value):
        """setter of desc"""
        check_var("desc", value, "str")
        self._desc = value

    desc = property(
        fget=_get_desc,
        fset=_set_desc,
        doc=u"""Simulation description

        :Type: str
        """,
    )

    def _get_machine(self):
        """getter of machine"""
        return self._machine

    def _set_machine(self, value):
        """setter of machine"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "machine"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Machine()
        check_var("machine", value, "Machine")
        self._machine = value

        if self._machine is not None:
            self._machine.parent = self

    machine = property(
        fget=_get_machine,
        fset=_set_machine,
        doc=u"""Machine to simulate

        :Type: Machine
        """,
    )

    def _get_input(self):
        """getter of input"""
        return self._input

    def _set_input(self, value):
        """setter of input"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "input"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Input()
        check_var("input", value, "Input")
        self._input = value

        if self._input is not None:
            self._input.parent = self

    input = property(
        fget=_get_input,
        fset=_set_input,
        doc=u"""Input of the simulation

        :Type: Input
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_var_simu(self):
        """getter of var_simu"""
        return self._var_simu

    def _set_var_simu(self, value):
        """setter of var_simu"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "var_simu"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = VarSimu()
        check_var("var_simu", value, "VarSimu")
        self._var_simu = value

        if self._var_simu is not None:
            self._var_simu.parent = self

    var_simu = property(
        fget=_get_var_simu,
        fset=_set_var_simu,
        doc=u"""Multi-simulation definition

        :Type: VarSimu
        """,
    )

    def _get_postproc_list(self):
        """getter of postproc_list"""
        if self._postproc_list is not None:
            for obj in self._postproc_list:
                if obj is not None:
                    obj.parent = self
        return self._postproc_list

    def _set_postproc_list(self, value):
        """setter of postproc_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "postproc_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
        if value == -1:
            value = list()
        check_var("postproc_list", value, "[Post]")
        self._postproc_list = value

    postproc_list = property(
        fget=_get_postproc_list,
        fset=_set_postproc_list,
        doc=u"""List of postprocessings to run on Output after the simulation

        :Type: [Post]
        """,
    )



# -*- coding: utf-8 -*-

from numpy import sin

from ....Classes.MagnetType10 import MagnetType10
from ....Classes.MagnetType11 import MagnetType11
from ....Classes.MagnetType13 import MagnetType13
from ....Classes.SlotMFlat import SlotMFlat
from ....Classes.SlotMPolar import SlotMPolar
from ....Classes.SlotMFlat import SlotMFlat
from ....Classes.SlotMPolar import SlotMPolar
from ....Methods.Machine.Lamination.check import LaminationCheckError


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self : LamSlotMag
        A LamSlotMag object

    Returns
    -------
    None

    Raises
    _______
    LMC_SlotTooLong
        The Slot is too long for the lamination (HYoke <0)
    LMC_OverlappingSlot
        The Lamination has overlapping slot
    LMC_MagnetShape
        You can't put a rounded bottom magnet in a flat bottom slot
    LMC_MagnetTooLarge
        The magnet is larger than the slot
    LMC_MagnetShape
        You can't put a flat bottom magnet in a rounded bottom slot
    """
    super(type(self), self).check()

    self.slot.check()

    if self.comp_height_yoke() < 0:
        raise LMC_SlotTooLong("The Slot is too long for the lamination " "(HYoke <0)")

    """
    Wt = self.slot.comp_tooth_widths()["WTooth_min"]

    if Wt < 0:
        raise LMC_OverlappingSlot("The Lamination has overlapping slot")
    """

    # Magnet check
    if isinstance(self.slot, SlotMFlat):
        if isinstance(self.magnet, MagnetType11):
            raise LMC_MagnetShape(
                "You can't put a rounded bottom magnet in " "a flat bottom slot"
            )
        if self.slot.W0_unit:  # W0 in rad
            W0 = self.Rext * sin(self.slot.W0) * 2
        else:  # W0 in [m]
            W0 = self.slot.W0
        if self.magnet.Wmag > W0:
            raise LMC_MagnetTooLarge("The magnet is larger than the slot")
    elif type(self.slot) in [SlotMPolar, SlotMPolar]:
        if type(self.magnet) in [MagnetType10, MagnetType13]:
            raise LMC_MagnetShape(
                "You can't put a flat bottom magnet in a " "rounded bottom slot"
            )
        if self.magnet.Wmag > self.slot.W0:
            raise LMC_MagnetTooLarge("The magnet is larger than the slot")
    elif isinstance(self.slot, SlotMFlat):
        if isinstance(self.magnet, MagnetType11):
            raise LMC_MagnetShape(
                "You can't put a rounded bottom magnet in " "a flat bottom slot"
            )
        if self.magnet.Wmag > self.slot.W0:
            raise LMC_MagnetTooLarge("The magnet is larger than the slot")
    else:  # Only is a new slot_mag type is added
        raise TypeError("Unknow Slot type")


class Lam_MagCheckError(LaminationCheckError):
    """ """

    pass


class LMC_OutSlotInLam(Lam_MagCheckError):
    """ """

    pass


class LMC_InSlotOutLam(Lam_MagCheckError):
    """ """

    pass


class LMC_SlotTooLong(Lam_MagCheckError):
    """ """

    pass


class LMC_OverlappingSlot(Lam_MagCheckError):
    """ """

    pass


class LMC_MagnetShape(Lam_MagCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    the
        slot

    """

    pass


class LMC_MagnetTooLarge(Lam_MagCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-


def get_cell_area(self, indices=None):
    """Return the area of the cells on the outer surface.

    Parameters
    ----------
    self : MeshMat
        a MeshMat object
    indices : list
        list of the points to extract (optional)

    Returns
    -------
    areas: ndarray
        Area of the cells
    """

    # TODO

    return area



# -*- coding: utf-8 -*-
from ....Functions.Geometry.inter_line_line import find_line_eq
from numpy import sqrt


def comp_distance(self, Z):
    """Compute the distance of a point to the Segment

    Parameters
    ----------
    self : Segment
        A Segment object
    Z : complex
        Complex coordinate of the point

    Returns
    -------
    D : float
        distance of a point to the Segment
    """

    Z1 = self.begin
    Z2 = self.end
    Z3 = Z
    if self.is_on_line(Z=Z):
        return 0
    # Check if the points are aligned
    Z12 = Z1 - Z2
    Z13 = Z1 - Z3
    if Z12.real * Z13.imag - Z12.imag * Z13.real == 0:
        return min(abs(Z1 - Z3), abs(Z2 - Z3))
    # point not aligned
    (A, B, C) = find_line_eq(Z1, Z2)
    return (abs(A * Z3.real + B * Z3.imag - C)) / sqrt(A ** 2 + B ** 2)



# -*- coding: utf-8 -*-

from ..init_fig import init_fig
from .plot_A_2D import plot_A_2D


def plot_A_quiver_2D(data, t=None, t_index=0, is_norm=False, unit="SI"):
    """Plots a 2D vector field

    Parameters
    ----------
    data : Data
        a Data object
    t : float
        time value at which to slice
    t_index : int
        time index at which to slice
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    """

    # Set plot
    (fig, axes, patch_leg, label_leg) = init_fig(None, shape="rectangle")
    xlabel = "x [m]"
    ylabel = "y [m]"
    # Prepare the extractions
    if t != None:
        t_str = "time=" + str(t)
    else:
        t_str = "time[" + str(t_index) + "]"
    title = data.name + " over space at " + t_str

    # Extract the fields
    results = data.get_along("xy", t_str, unit=unit, is_norm=is_norm)
    xy = results["xy"]
    Ydata = results[data.symbol]

    # Plot the original graph
    plot_A_2D(
        xy, [Ydata], fig=fig, title=title, xlabel=xlabel, ylabel=ylabel, type="quiver"
    )

    fig.show()



# -*- coding: utf-8 -*-


def check(self):
    """Make sure that the ventilation parameters are correctly set

    Parameters
    ----------
    self : VentilationCirc
        A VentilationCirc object

    Returns
    -------
    None

    Raises
    _______
    VentilationCircInstanceError
        Zh must be an integer
        H0 must be a float ot int
        D0 must be a float or int
        Alpha0 must be a float ot int
    """

    if not isinstance(self.Zh, int):
        raise VentilationCircInstanceError("Zh must be an integer")
    if not isinstance(self.H0, float) and not isinstance(self.H0, int):
        raise VentilationCircInstanceError("H0 must be a float ot int ")
    if not isinstance(self.D0, float) and not isinstance(self.D0, int):
        raise VentilationCircInstanceError("D0 must be a float or int ")
    if not isinstance(self.Alpha0, float) and not isinstance(self.Alpha0, int):
        raise VentilationCircInstanceError("Alpha0 must be a float ot int")


class VentilationCircInstanceError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-
from ....Methods.Machine.Winding import WindingError


def comp_Ntspc(self, Zs=None):
    """Compute the number of turns in series per phase

    Parameters
    ----------
    self : Winding
        A Winding object
    Zs : int
        Number of slot

    Returns
    -------
    Ntspc: float
        Number of turns in series per phase

    """
    if Zs is None:
        if self.parent is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object."
            )

        if self.parent.slot is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object with Slot."
            )

        Zs = self.parent.slot.Zs

    Ncspc = self.comp_Ncspc(Zs)

    return Ncspc * self.Ntcoil



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/SlotUD.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/SlotUD
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Slot import Slot

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.SlotUD.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error


from ._check import InitUnKnowClassError


class SlotUD(Slot):
    """"User defined" Slot from a point list. """

    VERSION = 1

    # cf Methods.Slot.SlotUD.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotUD method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, point_list=-1, is_sym=False, Zs=36, init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "point_list" in list(init_dict.keys()):
                point_list = init_dict["point_list"]
            if "is_sym" in list(init_dict.keys()):
                is_sym = init_dict["is_sym"]
            if "Zs" in list(init_dict.keys()):
                Zs = init_dict["Zs"]
        # Set the properties (value check and convertion are done in setter)
        self.point_list = point_list
        self.is_sym = is_sym
        # Call Slot init
        super(SlotUD, self).__init__(Zs=Zs)
        # The class is frozen (in Slot init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SlotUD_str = ""
        # Get the properties inherited from Slot
        SlotUD_str += super(SlotUD, self).__str__()
        SlotUD_str += (
            "point_list = "
            + linesep
            + str(self.point_list).replace(linesep, linesep + "\t")
            + linesep
        )
        SlotUD_str += "is_sym = " + str(self.is_sym) + linesep
        return SlotUD_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Slot
        if not super(SlotUD, self).__eq__(other):
            return False
        if other.point_list != self.point_list:
            return False
        if other.is_sym != self.is_sym:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from Slot
        SlotUD_dict = super(SlotUD, self).as_dict()
        SlotUD_dict["point_list"] = (
            self.point_list.copy() if self.point_list is not None else None
        )
        SlotUD_dict["is_sym"] = self.is_sym
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SlotUD_dict["__class__"] = "SlotUD"
        return SlotUD_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.point_list = None
        self.is_sym = None
        # Set to None the properties inherited from Slot
        super(SlotUD, self)._set_None()

    def _get_point_list(self):
        """getter of point_list"""
        return self._point_list

    def _set_point_list(self, value):
        """setter of point_list"""
        if type(value) is int and value == -1:
            value = list()
        check_var("point_list", value, "list")
        self._point_list = value

    point_list = property(
        fget=_get_point_list,
        fset=_set_point_list,
        doc=u"""Coordinates of the slot points (will be connected in order with Segments)

        :Type: list
        """,
    )

    def _get_is_sym(self):
        """getter of is_sym"""
        return self._is_sym

    def _set_is_sym(self, value):
        """setter of is_sym"""
        check_var("is_sym", value, "bool")
        self._is_sym = value

    is_sym = property(
        fget=_get_is_sym,
        fset=_set_is_sym,
        doc=u"""True to enter only half of the point coordinates

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Geometry/Surface.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Geometry/Surface
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Geometry.Surface.comp_mesh_dict import comp_mesh_dict
except ImportError as error:
    comp_mesh_dict = error

try:
    from ..Methods.Geometry.Surface.draw_FEMM import draw_FEMM
except ImportError as error:
    draw_FEMM = error

try:
    from ..Methods.Geometry.Surface.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Geometry.Surface.split_line import split_line
except ImportError as error:
    split_line = error


from ._check import InitUnKnowClassError


class Surface(FrozenClass):
    """SurfLine define by list of lines that delimit it, label and point reference."""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Geometry.Surface.comp_mesh_dict
    if isinstance(comp_mesh_dict, ImportError):
        comp_mesh_dict = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Surface method comp_mesh_dict: " + str(comp_mesh_dict)
                )
            )
        )
    else:
        comp_mesh_dict = comp_mesh_dict
    # cf Methods.Geometry.Surface.draw_FEMM
    if isinstance(draw_FEMM, ImportError):
        draw_FEMM = property(
            fget=lambda x: raise_(
                ImportError("Can't use Surface method draw_FEMM: " + str(draw_FEMM))
            )
        )
    else:
        draw_FEMM = draw_FEMM
    # cf Methods.Geometry.Surface.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Surface method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Geometry.Surface.split_line
    if isinstance(split_line, ImportError):
        split_line = property(
            fget=lambda x: raise_(
                ImportError("Can't use Surface method split_line: " + str(split_line))
            )
        )
    else:
        split_line = split_line
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, point_ref=0, label="", init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "point_ref" in list(init_dict.keys()):
                point_ref = init_dict["point_ref"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.point_ref = point_ref
        self.label = label

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Surface_str = ""
        if self.parent is None:
            Surface_str += "parent = None " + linesep
        else:
            Surface_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Surface_str += "point_ref = " + str(self.point_ref) + linesep
        Surface_str += 'label = "' + str(self.label) + '"' + linesep
        return Surface_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.point_ref != self.point_ref:
            return False
        if other.label != self.label:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        Surface_dict = dict()
        Surface_dict["point_ref"] = self.point_ref
        Surface_dict["label"] = self.label
        # The class name is added to the dict for deserialisation purpose
        Surface_dict["__class__"] = "Surface"
        return Surface_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.point_ref = None
        self.label = None

    def _get_point_ref(self):
        """getter of point_ref"""
        return self._point_ref

    def _set_point_ref(self, value):
        """setter of point_ref"""
        check_var("point_ref", value, "complex")
        self._point_ref = value

    point_ref = property(
        fget=_get_point_ref,
        fset=_set_point_ref,
        doc=u"""Center of symmetry

        :Type: complex
        """,
    )

    def _get_label(self):
        """getter of label"""
        return self._label

    def _set_label(self, value):
        """setter of label"""
        check_var("label", value, "str")
        self._label = value

    label = property(
        fget=_get_label,
        fset=_set_label,
        doc=u"""Label of the surface

        :Type: str
        """,
    )



class VarSimuError(Exception):
    pass


def get_simulations(self):
    """Check VarParam parameters validity"""
    raise VarSimuError(
        "VarSimu is an abstract class, please create one of its daughters."
    )



from ....Classes.SolutionData import SolutionData
from ....Classes.SolutionVector import SolutionVector
from ....Classes.MeshSolution import MeshSolution
from SciDataTool import DataTime, Data1D, VectorField
import numpy as np


def build_meshsolution(self, Nt_tot, meshFEMM, Time, B, H, mu, groups):
    """Build the MeshSolution objets from FEMM outputs.

    Parameters
    ----------
    self : MagFEMM
        a MagFEMM object
    is_get_mesh : bool
        1 to load the mesh and solution into the simulation
    is_save_FEA : bool
        1 to save the mesh and solution into a .json file
    j_t0 : int
        Targeted time step

    Returns
    -------
    meshsol: MeshSolution
        a MeshSolution object with FEMM outputs at every time step
    """

    sollist = list()
    cond = self.is_sliding_band or Nt_tot == 1
    if cond:
        indices_cell = meshFEMM[0].cell["triangle"].indice
        Direction = Data1D(name="direction", values=["x", "y", "z"], is_components=True)
        Indices_Cell = Data1D(name="indice", values=indices_cell, is_components=True)
        Nodirection = Data1D(name="direction", values=["scalar"], is_components=False)

        # Store the results for B
        components = {}

        Bx_data = DataTime(
            name="Magnetic Flux Density Bx",
            unit="T",
            symbol="Bx",
            axes=[Time, Indices_Cell],
            values=B[:, :, 0],
        )
        components["x"] = Bx_data

        By_data = DataTime(
            name="Magnetic Flux Density By",
            unit="T",
            symbol="By",
            axes=[Time, Indices_Cell],
            values=B[:, :, 1],
        )
        components["y"] = By_data

        if not np.all((B[:, :, 2] == 0)):
            Bz_data = DataTime(
                name="Magnetic Flux Density Bz",
                unit="T",
                symbol="Bz",
                axes=[Time, Indices_Cell],
                values=B[:, :, 2],
            )
            components["z"] = Bz_data

        solB = VectorField(
            name="Magnetic Flux Density", symbol="B", components=components
        )

        # Store the results for H
        componentsH = {}

        Hx_data = DataTime(
            name="Magnetic Field Hx",
            unit="A/m",
            symbol="Hx",
            axes=[Time, Indices_Cell],
            values=H[:, :, 0],
        )
        componentsH["x"] = Hx_data

        Hy_data = DataTime(
            name="Magnetic Field Hy",
            unit="A/m",
            symbol="Hy",
            axes=[Time, Indices_Cell],
            values=H[:, :, 1],
        )
        componentsH["y"] = Hy_data

        if not np.all((H[:, :, 2] == 0)):
            Hz_data = DataTime(
                name="Magnetic Field Hz",
                unit="A/m",
                symbol="Hz",
                axes=[Time, Indices_Cell],
                values=H[:, :, 2],
            )
            componentsH["z"] = Hz_data

        solH = VectorField(name="Magnetic Field", symbol="H", components=componentsH)

        solmu = DataTime(
            name="Magnetic Permeability",
            unit="H/m",
            symbol="\mu",
            axes=[Time, Indices_Cell],
            values=mu,
        )

        sollist.append(
            SolutionVector(field=solB, type_cell="triangle", label="B")
        )  # Face solution
        sollist.append(SolutionVector(field=solH, type_cell="triangle", label="H"))
        sollist.append(SolutionData(field=solmu, type_cell="triangle", label="\mu"))

    meshsol = MeshSolution(
        label="FEMM_magnetotatic",
        mesh=meshFEMM,
        solution=sollist,
        is_same_mesh=cond,
        dimension=2,
    )

    meshsol.group = groups[0]

    return meshsol



# -*- coding: utf-8 -*-


def comp_time_angle(self, output):
    """Compute the time and space discretization of the Force module

    Parameters
    ----------
    self : Force
        a Force object
    output : Output
        an Output object (to update)
    """

    output.force.time = output.mag.time
    output.force.angle = output.mag.angle



# -*- coding: utf-8 -*-
"""File generated according to PMagnet14/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SMagnet.PMagnet14.Ui_PMagnet14 import (
    Ui_PMagnet14,
)


class Gen_PMagnet14(Ui_PMagnet14):
    def setupUi(self, PMagnet14):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PMagnet14.setupUi(self, PMagnet14)
        # Setup of in_Hmag
        txt = self.tr(u"""magnet radial height [m]""")
        self.in_Hmag.setWhatsThis(txt)
        self.in_Hmag.setToolTip(txt)

        # Setup of lf_Hmag
        self.lf_Hmag.validator().setBottom(0)
        txt = self.tr(u"""magnet radial height [m]""")
        self.lf_Hmag.setWhatsThis(txt)
        self.lf_Hmag.setToolTip(txt)

        # Setup of in_Wmag
        txt = self.tr(u"""magnet bottom width [rad]""")
        self.in_Wmag.setWhatsThis(txt)
        self.in_Wmag.setToolTip(txt)

        # Setup of lf_Wmag
        self.lf_Wmag.validator().setBottom(0)
        txt = self.tr(u"""magnet bottom width [rad]""")
        self.lf_Wmag.setWhatsThis(txt)
        self.lf_Wmag.setToolTip(txt)

        # Setup of in_Rtopm
        txt = self.tr(u"""radius of the circular top shape [m]""")
        self.in_Rtopm.setWhatsThis(txt)
        self.in_Rtopm.setToolTip(txt)

        # Setup of lf_Rtopm
        self.lf_Rtopm.validator().setBottom(0)
        txt = self.tr(u"""radius of the circular top shape [m]""")
        self.lf_Rtopm.setWhatsThis(txt)
        self.lf_Rtopm.setToolTip(txt)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/MagnetFlat.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/MagnetFlat
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Magnet import Magnet

from ._check import InitUnKnowClassError
from .Material import Material


class MagnetFlat(Magnet):
    """abstract class of magnets with rectangular base"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        mat_type=-1,
        type_magnetization=0,
        Lmag=0.95,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "mat_type" in list(init_dict.keys()):
                mat_type = init_dict["mat_type"]
            if "type_magnetization" in list(init_dict.keys()):
                type_magnetization = init_dict["type_magnetization"]
            if "Lmag" in list(init_dict.keys()):
                Lmag = init_dict["Lmag"]
        # Set the properties (value check and convertion are done in setter)
        # Call Magnet init
        super(MagnetFlat, self).__init__(
            mat_type=mat_type, type_magnetization=type_magnetization, Lmag=Lmag
        )
        # The class is frozen (in Magnet init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MagnetFlat_str = ""
        # Get the properties inherited from Magnet
        MagnetFlat_str += super(MagnetFlat, self).__str__()
        return MagnetFlat_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Magnet
        if not super(MagnetFlat, self).__eq__(other):
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from Magnet
        MagnetFlat_dict = super(MagnetFlat, self).as_dict()
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        MagnetFlat_dict["__class__"] = "MagnetFlat"
        return MagnetFlat_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        # Set to None the properties inherited from Magnet
        super(MagnetFlat, self)._set_None()



# -*- coding: utf-8 -*-

from numpy import linalg as LA, pi, sign, sqrt


def create_FEMM_circuit(femm, label, is_eddies, lam, I, is_mmf, j_t0, materials):
    """Set in FEMM circuits property

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    label :
        label of the surface
    sym :
        integer for symmetry
    is_eddies :
        1 to calculate eddy currents
    lam : LamSlotWind
        Lamination to set the winding
    I : ndarray
        Lamination currents waveforms
    is_mmf :
        1 to compute the lamination magnetomotive
        force / lamination magnetic field
    j_t0 :
        time step for winding current calculation@type integer
    materials :
        list of materials already created in FEMM

    Returns
    -------
    (str, list)
        the property of the surface having label as surf.label and
        materials
    """

    is_defcirc = 1  # 1 to define circuits (necessary for eddy current loss
    kJ = 1
    # estimation in AC mode, not necessary for magnetostatics)
    Zs = lam.slot.Zs
    rho = lam.mat_type.elec.rho  # Resistivity at 20°C
    wind_mat = lam.winding.comp_connection_mat(Zs)
    # number of parallel circuits  per phase (maximum 2p)
    Npcpp = lam.winding.Npcpp
    Ntcoil = lam.winding.Ntcoil  # number of turns per coil
    Swire = lam.winding.conductor.comp_surface()
    Sact = lam.winding.conductor.comp_active_surface()
    if "Rotor" in label:  # winding on the rotor
        Jlabel = "Jr"
        Clabel = "Circr"
    else:
        Jlabel = "Js"
        Clabel = "Circs"

    st = label.split("_")
    Nr = int(st[1][1:])  # zone radial coordinate
    Nt = int(st[2][1:])  # zone tangential coordinate
    pos = int(st[3][1:])  # Zone slot number coordinate
    A = wind_mat[Nr, Nt, pos, :]

    for zz in range(len(A)):  # search  for the phase of the winding
        if A[zz] != 0:
            q = zz
    # circuit definition
    if is_defcirc:
        if I.size == 0 or LA.norm(I) == 0:
            femm.mi_addcircprop(Clabel + str(q), 0, 1)  # series connected
        else:
            femm.mi_addcircprop(
                Clabel + str(q), is_mmf * I[j_t0][q] / Npcpp, 1
            )  # series connected

    # definition of armature field current sources and  circuits
    s = sign(wind_mat[Nr, Nt, pos, q])  # same as the sign of Cwind1
    if s == 1:
        cname = Jlabel + str(q) + "+"
    else:
        cname = Jlabel + str(q) + "-"

    if LA.norm(I) == 0:
        Jcus = 0
    else:
        # equivalent stator current density [A/mm2]
        Jcus = kJ * 1e-6 * Ntcoil * (I[j_t0, q] / Npcpp) * is_mmf / Sact

    if cname not in materials:
        # adding new current source material if necessary
        femm.mi_addmaterial(
            cname,
            1,
            1,
            0,
            s * Jcus,
            is_eddies * 1e-6 / rho,
            0,
            0,
            1,
            0,
            0,
            0,
            1,
            sqrt(4 * Swire / pi),
        )
        materials.append(cname)

    return cname, materials



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/VentilationCirc.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/VentilationCirc
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Hole import Hole

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.VentilationCirc.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Slot.VentilationCirc.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Slot.VentilationCirc.comp_radius import comp_radius
except ImportError as error:
    comp_radius = error

try:
    from ..Methods.Slot.VentilationCirc.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Slot.VentilationCirc.get_center import get_center
except ImportError as error:
    get_center = error


from ._check import InitUnKnowClassError
from .Material import Material


class VentilationCirc(Hole):
    """Circular axial ventilation duct"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.VentilationCirc.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VentilationCirc method build_geometry: "
                    + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Slot.VentilationCirc.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use VentilationCirc method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Slot.VentilationCirc.comp_radius
    if isinstance(comp_radius, ImportError):
        comp_radius = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VentilationCirc method comp_radius: " + str(comp_radius)
                )
            )
        )
    else:
        comp_radius = comp_radius
    # cf Methods.Slot.VentilationCirc.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VentilationCirc method comp_surface: "
                    + str(comp_surface)
                )
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Slot.VentilationCirc.get_center
    if isinstance(get_center, ImportError):
        get_center = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VentilationCirc method get_center: " + str(get_center)
                )
            )
        )
    else:
        get_center = get_center
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, Alpha0=0, D0=1, H0=1, Zh=36, mat_void=-1, init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Alpha0" in list(init_dict.keys()):
                Alpha0 = init_dict["Alpha0"]
            if "D0" in list(init_dict.keys()):
                D0 = init_dict["D0"]
            if "H0" in list(init_dict.keys()):
                H0 = init_dict["H0"]
            if "Zh" in list(init_dict.keys()):
                Zh = init_dict["Zh"]
            if "mat_void" in list(init_dict.keys()):
                mat_void = init_dict["mat_void"]
        # Set the properties (value check and convertion are done in setter)
        self.Alpha0 = Alpha0
        self.D0 = D0
        self.H0 = H0
        # Call Hole init
        super(VentilationCirc, self).__init__(Zh=Zh, mat_void=mat_void)
        # The class is frozen (in Hole init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        VentilationCirc_str = ""
        # Get the properties inherited from Hole
        VentilationCirc_str += super(VentilationCirc, self).__str__()
        VentilationCirc_str += "Alpha0 = " + str(self.Alpha0) + linesep
        VentilationCirc_str += "D0 = " + str(self.D0) + linesep
        VentilationCirc_str += "H0 = " + str(self.H0) + linesep
        return VentilationCirc_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Hole
        if not super(VentilationCirc, self).__eq__(other):
            return False
        if other.Alpha0 != self.Alpha0:
            return False
        if other.D0 != self.D0:
            return False
        if other.H0 != self.H0:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from Hole
        VentilationCirc_dict = super(VentilationCirc, self).as_dict()
        VentilationCirc_dict["Alpha0"] = self.Alpha0
        VentilationCirc_dict["D0"] = self.D0
        VentilationCirc_dict["H0"] = self.H0
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        VentilationCirc_dict["__class__"] = "VentilationCirc"
        return VentilationCirc_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Alpha0 = None
        self.D0 = None
        self.H0 = None
        # Set to None the properties inherited from Hole
        super(VentilationCirc, self)._set_None()

    def _get_Alpha0(self):
        """getter of Alpha0"""
        return self._Alpha0

    def _set_Alpha0(self, value):
        """setter of Alpha0"""
        check_var("Alpha0", value, "float", Vmin=0, Vmax=6.29)
        self._Alpha0 = value

    Alpha0 = property(
        fget=_get_Alpha0,
        fset=_set_Alpha0,
        doc=u"""Shift angle of the holes around circumference

        :Type: float
        :min: 0
        :max: 6.29
        """,
    )

    def _get_D0(self):
        """getter of D0"""
        return self._D0

    def _set_D0(self, value):
        """setter of D0"""
        check_var("D0", value, "float", Vmin=0)
        self._D0 = value

    D0 = property(
        fget=_get_D0,
        fset=_set_D0,
        doc=u"""Hole diameters

        :Type: float
        :min: 0
        """,
    )

    def _get_H0(self):
        """getter of H0"""
        return self._H0

    def _set_H0(self, value):
        """setter of H0"""
        check_var("H0", value, "float", Vmin=0)
        self._H0 = value

    H0 = property(
        fget=_get_H0,
        fset=_set_H0,
        doc=u"""Diameter of the hole centers

        :Type: float
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-

from numpy import array


def comp_height(self):
    """Compute the height of the Magnet.
    Caution, the bottom of the Magnet is an Arc

    Parameters
    ----------
    self : Magnet
        A Magnet object

    Returns
    -------
    Htot: float
        Height of the Magnet [m]

    """

    surf = self.build_geometry()

    # Numerical computation
    point_list = surf[0].discretize(200)
    point_list = array(point_list)
    abs_list = abs(point_list)

    return max(abs_list) - min(abs_list)



# -*- coding: utf-8 -*-
from numpy import arcsin, exp


def get_point_bottom(self):
    """Used to get the bottoms points of a single SlotMPolar

    Parameters
    ----------
    self : SlotMPolar
        A SlotMPolar object


    Returns
    -------
    (Z1, Z2) : tuple
        points at the bottom of the SlotMPolar

    """

    Rbo = self.get_Rbo()

    if self.is_outwards():
        Z1 = (Rbo + self.H0) * exp(-1j * self.W0 / 2)
        Z2 = (Rbo + self.H0) * exp(1j * self.W0 / 2)
    else:
        Z1 = (Rbo - self.H0) * exp(-1j * self.W0 / 2)
        Z2 = (Rbo - self.H0) * exp(1j * self.W0 / 2)

    return (Z1, Z2)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Material/MatEconomical.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Material/MatEconomical
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError


class MatEconomical(FrozenClass):
    """material ecomomical properties"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, cost_unit=0.127, unit_name="$", init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "cost_unit" in list(init_dict.keys()):
                cost_unit = init_dict["cost_unit"]
            if "unit_name" in list(init_dict.keys()):
                unit_name = init_dict["unit_name"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.cost_unit = cost_unit
        self.unit_name = unit_name

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MatEconomical_str = ""
        if self.parent is None:
            MatEconomical_str += "parent = None " + linesep
        else:
            MatEconomical_str += (
                "parent = " + str(type(self.parent)) + " object" + linesep
            )
        MatEconomical_str += "cost_unit = " + str(self.cost_unit) + linesep
        MatEconomical_str += 'unit_name = "' + str(self.unit_name) + '"' + linesep
        return MatEconomical_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.cost_unit != self.cost_unit:
            return False
        if other.unit_name != self.unit_name:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        MatEconomical_dict = dict()
        MatEconomical_dict["cost_unit"] = self.cost_unit
        MatEconomical_dict["unit_name"] = self.unit_name
        # The class name is added to the dict for deserialisation purpose
        MatEconomical_dict["__class__"] = "MatEconomical"
        return MatEconomical_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.cost_unit = None
        self.unit_name = None

    def _get_cost_unit(self):
        """getter of cost_unit"""
        return self._cost_unit

    def _set_cost_unit(self, value):
        """setter of cost_unit"""
        check_var("cost_unit", value, "float", Vmin=0)
        self._cost_unit = value

    cost_unit = property(
        fget=_get_cost_unit,
        fset=_set_cost_unit,
        doc=u"""Cost of one kilo of material

        :Type: float
        :min: 0
        """,
    )

    def _get_unit_name(self):
        """getter of unit_name"""
        return self._unit_name

    def _set_unit_name(self, value):
        """setter of unit_name"""
        check_var("unit_name", value, "str")
        self._unit_name = value

    unit_name = property(
        fget=_get_unit_name,
        fset=_set_unit_name,
        doc=u"""Name of the unit

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-

from os.path import join, dirname, abspath


def get_path_result(self):
    """Return the path to the result folder. If there is a name for the simulation, the path leads to a sub-folder of
    'Results' folder which has the same name. If this sub-folder does not exist, it creates it.

    Parameters
    ----------
    self : Output
        an Output object

    Returns
    -------
    res_path: str
        path to the result folder
    """

    if self.path_res in [None, ""]:
        # If the path doesn't exist, set the default one
        self.path_res = abspath(
            join(dirname(__file__), "..", "..", "..", "..", "Results", self.simu.name)
        )
    return self.path_res



# -*- coding: utf-8 -*-

from .....Functions.Plot.plot_A_quiver_2D import (
    plot_A_quiver_2D as plot_A_quiver_2D_fct,
)


def plot_A_quiver_2D(self, Data_str, t=None, t_index=0, is_norm=False, unit="SI"):
    """Plots a field as a function of space (angle)

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data object to plot (e.g. "mag.Br")
    t : float
        time value at which to slice
    t_index : int
        time index at which to slice
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    """

    # Get Data object names
    phys = getattr(self, Data_str.split(".")[0])
    data = getattr(phys, Data_str.split(".")[1])

    # Call the plot function
    plot_A_quiver_2D_fct(data, t=t, t_index=t_index, is_norm=is_norm, unit=unit)



# -*- coding: utf-8 -*-
from numpy import pi, angle, exp

from ....Classes.Circle import Circle
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfRing import SurfRing
from ....Methods.Machine.Lamination.build_geometry import build_geometry as build_geo


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the LamSlot object

    Parameters
    ----------
    self : LamSlot
        a LamSlot object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """

    if self.is_stator:
        label = "Lamination_Stator"  # Label lamination
    else:
        label = "Lamination_Rotor"
    if self.is_internal:
        lb = "Ext"  # label for the bore
        ly = "Int"  # label for the yoke
    else:
        lb = "Int"
        ly = "Ext"
    label_bore = label + "_Bore_Radius_" + lb
    label_yoke = label + "_Yoke_Radius_" + ly

    Ryoke = self.get_Ryoke()
    H_yoke = self.comp_height_yoke()

    if self.slot is not None and self.slot.Zs != 0:
        # getting Number of Slot
        Zs = self.slot.Zs

        # Check for symmetry
        assert (Zs % sym) == 0, (
            "ERROR, Wrong symmetry for "
            + label
            + " "
            + str(Zs)
            + " slots and sym="
            + str(sym)
        )

        bore_desc = self.get_bore_desc(sym=sym)
        bore_list = list()
        for bore in bore_desc:
            if type(bore["obj"]) is Arc1:
                bore_list.append(bore["obj"])
            elif "lines" in bore:  # Duplicated slot
                for line in bore["lines"]:
                    bore_list.append(type(line)(init_dict=line.as_dict()))
                    bore_list[-1].rotate((bore["begin_angle"] + bore["end_angle"]) / 2)
            else:  # Notches
                lines = bore["obj"].build_geometry()
                for line in lines:
                    line.rotate((bore["begin_angle"] + bore["end_angle"]) / 2)
                bore_list.extend(lines)
        # Set line label
        for line in bore_list:
            line.label = label_bore
    else:  # No slot
        return build_geo(self, sym=sym, alpha=alpha, delta=delta)

    # Create the lamination surface(s)
    surf_list = list()
    if sym == 1:  # Complete lamination
        if self.is_internal:
            point_ref = Ryoke + (H_yoke / 2)
        else:
            point_ref = Ryoke - (H_yoke / 2)
        # Create Slot surface
        surf_slot = SurfLine(line_list=bore_list, label=label_bore, point_ref=point_ref)
        # Create yoke circle surface
        surf_yoke = Circle(
            radius=Ryoke, label=label_yoke, line_label=label_yoke, center=0
        )
        if self.Rint == 0:
            surf_list = [surf_slot]
        elif self.is_internal:
            surf_list.append(
                SurfRing(
                    out_surf=surf_slot,
                    in_surf=surf_yoke,
                    label=label,
                    point_ref=point_ref,
                )
            )
        else:
            surf_list.append(
                SurfRing(
                    out_surf=surf_yoke,
                    in_surf=surf_slot,
                    label=label,
                    point_ref=point_ref,
                )
            )
    else:  # Only one surface
        # Add the Yoke part
        Zy1 = Ryoke
        Zy2 = Ryoke * exp(1j * 2 * pi / sym)
        bore_list.append(
            Segment(bore_list[-1].get_end(), Zy2, label=label + "_Yoke_Side")
        )
        if Ryoke > 0:  # For internal lamination
            bore_list.append(
                Arc1(
                    begin=Zy2,
                    end=Zy1,
                    radius=-Ryoke,
                    is_trigo_direction=False,
                    label=label_yoke,
                )
            )
        bore_list.append(
            Segment(Zy1, bore_list[0].get_begin(), label=label + "_Yoke_Side")
        )
        # Create a Surface for the slot
        if self.is_internal:
            point_ref = (Ryoke + H_yoke / 2) * exp(1j * pi / sym)
        else:
            point_ref = (Ryoke - H_yoke / 2) * exp(1j * pi / sym)
        surf_slot = SurfLine(line_list=bore_list, label=label_bore, point_ref=point_ref)
        surf_list.append(surf_slot)

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    # Adding the ventilation surfaces
    for vent in self.axial_vent:
        surf_list += vent.build_geometry(
            sym=sym, alpha=alpha, delta=delta, is_stator=self.is_stator
        )
    return surf_list



import numpy as np
import itertools
from ....Functions.Simulation.VarSimu.run_single_simu import run_single_simu


def run(self):
    """Run each simulation contained"""
    # Check var_simu parameters
    self.check_param()

    # Get xoutput to store results
    xoutput = self.parent.parent

    # Extact simulation list and ParamExplorerValue list
    simu_dict = self.get_simulations()

    # Fill input_parameters
    xoutput.nb_simu = simu_dict["nb_simu"]
    xoutput.paramexplorer_list = simu_dict["paramexplorer_list"]
    simulation_list = simu_dict["simulation_list"]

    # Construct results
    for datakeeper in self.datakeeper_list:
        xoutput.xoutput_dict[datakeeper.symbol] = datakeeper
        datakeeper.result = [None] * self.nb_simu

    # Execute the reference simulation it is included in the simulation list
    # Otherwise, the reference simulation is already executed in the simulation.run method
    nb_simu = self.nb_simu
    ref_simu_index = self.ref_simu_index
    index_list = list(range(nb_simu))

    ref_simu_in_multsim = isinstance(self.ref_simu_index, int)

    if ref_simu_in_multsim:
        logger = self.get_logger()
        logger.info("Computing reference simulation")

        simulation = simulation_list.pop(ref_simu_index)
        index_list.pop(ref_simu_index)
        xoutput.simu = simulation

        # Run the simulation handling errors
        run_single_simu(
            xoutput,
            self.datakeeper_list,
            simulation,
            ref_simu_index,
            self.stop_if_error,
            self.ref_simu_index,
            self.is_keep_all_output,
        )

        # Set back the var_simu
        simulation.var_simu = self
        print(
            "\r["
            + "=" * (50 * (1) // (nb_simu))
            + " " * (50 - ((50) // (nb_simu)))
            + "] {:3d}%".format(((100 * 1) // (nb_simu)))
        )

    # Reuse some intermediate results from reference simulation (if requested)
    for simu in simulation_list:
        self.set_reused_data(simu, xoutput)

    # Execute the other simulations
    nb_simu = self.nb_simu
    for idx, [i, simulation] in zip(index_list, enumerate(simulation_list)):
        # Run the simulation handling errors
        run_single_simu(
            xoutput,
            self.datakeeper_list,
            simulation,
            idx,
            self.stop_if_error,
            self.ref_simu_index,
            self.is_keep_all_output,
        )

        print(
            "\r["
            + "=" * (50 * (i + 1 + ref_simu_in_multsim) // (nb_simu))
            + " " * (50 - ((50 * (i + 1 + ref_simu_in_multsim)) // (nb_simu)))
            + "] {:3d}%".format(((100 * (i + 1 + ref_simu_in_multsim)) // (nb_simu))),
            end="",
        )

    # Running postprocessings
    if self.postproc_list:
        logger.info("Running var_simu postprocessings...")
        for postproc in self.postproc_list:
            postproc.run(xoutput)



from PySide2.QtCore import QSize
from PySide2.QtWidgets import QGroupBox, QLabel, QVBoxLayout, QWidget

from ......GUI import gui_option


class WMagnetOut(QGroupBox):
    """Setup of QGroupBox for output for Magnet"""

    def __init__(self, parent=None):
        """Initialize the widget"""

        QGroupBox.__init__(self, parent)
        self.u = gui_option.unit
        # Setup main widget
        self.setTitle(self.tr("Output"))
        self.setMinimumSize(QSize(200, 0))
        self.setObjectName("g_output")
        self.layout = QVBoxLayout(self)
        self.layout.setObjectName("layout")
        # The widget is composed of several QLabel in a vertical layout
        self.out_Smag = QLabel(self)
        self.out_Smag.setObjectName("out_Smag")
        self.layout.addWidget(self.out_Smag)

        self.out_gap = QLabel(self)
        self.out_gap.setObjectName("out_gap")
        self.layout.addWidget(self.out_gap)

        self.out_gap_min = QLabel(self)
        self.out_gap_min.setObjectName("out_gap_min")
        self.layout.addWidget(self.out_gap_min)

        self.out_taum = QLabel(self)
        self.out_taum.setObjectName("out_taum")
        self.layout.addWidget(self.out_taum)

    def comp_output(self):
        """Update the Output text with the computed values

        Parameters
        ----------
        self : WMagnetOut
            A WMagnetOut object
        """
        mach = self.parent().machine
        # Gap is set in SMachineDimension
        gap = format(self.u.get_m(mach.comp_width_airgap_mag()), ".4g")
        self.out_gap.setText(self.tr("gap: ") + gap + " " + self.u.get_m_name())

        mag_txt = self.tr("Magnet surface: ")
        gm_txt = self.tr("gap_min: ")
        taum_txt = self.tr("taum: ")

        if self.parent().check() is None:
            # We compute the output only if the slot is correctly set
            Zs = mach.rotor.slot.Zs
            # Compute all the needed output as string (scientific notation with
            # 2 digits)
            Smag = format(
                self.u.get_m2(mach.rotor.slot.magnet[0].comp_surface()), ".4g"
            )
            gap_min = format(self.u.get_m(mach.comp_width_airgap_mec()), ".4g")

            taum = 100 * mach.rotor.slot.magnet[0].comp_ratio_opening(Zs / 2.0)
            taum = "%.4g" % (taum)

            # Update the GUI to display the Output
            self.out_Smag.setText(mag_txt + Smag + " " + self.u.get_m2_name())
            self.out_gap_min.setText(gm_txt + gap_min + " " + self.u.get_m_name())
            self.out_taum.setText(taum_txt + taum + " %")
        else:
            # We can't compute the output => We erase the previous version
            # (that way the user know that something is wrong without
            # clicking next)
            self.out_Smag.setText(mag_txt + "?")
            self.out_gap_min.setText(gm_txt + "?")
            self.out_taum.setText(taum_txt + " ?")



# -*- coding: utf-8 -*-
from numpy import pi

from ....Methods.Machine.LamSlot.build_geometry import build_geometry as build_geo


def build_geometry(self, is_magnet=True, sym=1, alpha=0, delta=0, is_simplified=False):
    """Build the geometry of the LamSlotMag

    Parameters
    ----------
    self : LamSlotMag
        LamSlotMag object
    is_magnet : bool
        If True build the magnet surfaces
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """

    if self.is_stator:
        st = "Stator"
    else:
        st = "Rotor"

    assert (self.slot.Zs % sym) == 0, (
        "ERROR, Wrong symmetry for "
        + st
        + " "
        + str(self.slot.Zs)
        + " slots and sym="
        + str(sym)
    )
    # getting the LamSlot surface
    surf_list = build_geo(self, sym=sym)

    Zs = self.slot.Zs
    slot_pitch = 2 * pi / Zs
    Nmag = len(self.slot.magnet)
    Sw = self.slot.comp_angle_opening()  # Opening angle of all the slots
    Sw_mag = self.slot.comp_angle_opening_magnet()  # Opening angle of one slot

    # Add the magnet surface(s)
    if is_magnet:
        # for each slot to draw
        for ii in range(Zs // sym):
            alpha_s = ii * slot_pitch + slot_pitch / 2  # Angle to rotate
            # the magnet
            for jj in range(Nmag):  # for each subslot in the slot
                beta = alpha_s - Sw / 2 + Sw_mag / 2 + (Sw_mag + self.slot.W3) * jj
                mag_surf = self.slot.magnet[jj].build_geometry(
                    alpha=beta, is_simplified=is_simplified
                )
                # Defining type of magnetization of the magnet
                if self.slot.magnet[jj].type_magnetization == 0:
                    type_mag = "Radial"
                elif self.slot.magnet[jj].type_magnetization == 1:
                    type_mag = "Parallel"
                elif self.slot.magnet[jj].type_magnetization == 2:
                    type_mag = "Hallbach"

                surf_list.extend(mag_surf)
                # Adapt the label
                if ii % 2 != 0:  # South pole
                    surf_list[-1].label = (
                        "Magnet"
                        + st
                        + type_mag
                        + "_S_R0"
                        + "_T"
                        + str(jj)
                        + "_S"
                        + str(ii)
                    )
                else:  # North pole
                    surf_list[-1].label = (
                        "Magnet"
                        + st
                        + type_mag
                        + "_N_R0"
                        + "_T"
                        + str(jj)
                        + "_S"
                        + str(ii)
                    )

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-

from numpy import exp, pi, sqrt

from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : HoleM58
        A HoleM58 object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list: list
        List of SurfLine needed to draw the HoleM58

    """

    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"
    Rbo = self.get_Rbo()

    Z0 = Rbo - self.H0
    Z2 = Z0 + 1j * (self.W0 / 2)
    Z1 = Z2 - 1j * self.W2
    Z12 = Z1 - 1j * self.W1
    Z11 = Z2 - 1j * self.W0
    Z5 = Z2 - self.H2
    Z6 = Z5 - 1j * self.W2
    Z7 = Z6 - 1j * self.W1
    Z8 = Z5 - 1j * self.W0

    Zc1 = (Rbo - self.H1 - self.R0) * exp(1j * self.W3 / 2)

    # Z3 is the tangent point of the circle for (Z3,Z2)
    # (X3 - X2) * (X3 - Xc) + (Y3 - Y2) * (Y3 - Yc)
    # Z3 is on the circle
    # (X3 - Xc) ** 2 + (Y3 - Yc) ** 2 - R0 ** 2
    R0 = self.R0
    X2 = Z2.real
    Y2 = Z2.imag
    Xc = Zc1.real
    Yc = Zc1.imag
    # Solved with Sympy
    X3 = (
        R0 ** 2 * X2
        - R0 ** 2 * Xc
        - R0
        * Y2
        * sqrt(
            -(R0 ** 2)
            + X2 ** 2
            - 2 * X2 * Xc
            + Xc ** 2
            + Y2 ** 2
            - 2 * Y2 * Yc
            + Yc ** 2
        )
        + R0
        * Yc
        * sqrt(
            -(R0 ** 2)
            + X2 ** 2
            - 2 * X2 * Xc
            + Xc ** 2
            + Y2 ** 2
            - 2 * Y2 * Yc
            + Yc ** 2
        )
        + X2 ** 2 * Xc
        - 2 * X2 * Xc ** 2
        + Xc ** 3
        + Xc * Y2 ** 2
        - 2 * Xc * Y2 * Yc
        + Xc * Yc ** 2
    ) / (X2 ** 2 - 2 * X2 * Xc + Xc ** 2 + Y2 ** 2 - 2 * Y2 * Yc + Yc ** 2)
    Y3 = (
        R0 ** 2 * Y2
        - R0 ** 2 * Yc
        + R0
        * (X2 - Xc)
        * sqrt(
            -(R0 ** 2)
            + X2 ** 2
            - 2 * X2 * Xc
            + Xc ** 2
            + Y2 ** 2
            - 2 * Y2 * Yc
            + Yc ** 2
        )
        + X2 ** 2 * Yc
        - 2 * X2 * Xc * Yc
        + Xc ** 2 * Yc
        + Y2 ** 2 * Yc
        - 2 * Y2 * Yc ** 2
        + Yc ** 3
    ) / (X2 ** 2 - 2 * X2 * Xc + Xc ** 2 + Y2 ** 2 - 2 * Y2 * Yc + Yc ** 2)
    Z3 = X3 + 1j * Y3

    # Same for Z4
    X5 = Z5.real
    Y5 = Z5.imag
    X4 = (
        R0 ** 2 * X5
        - R0 ** 2 * Xc
        + R0
        * Y5
        * sqrt(
            -(R0 ** 2)
            + X5 ** 2
            - 2 * X5 * Xc
            + Xc ** 2
            + Y5 ** 2
            - 2 * Y5 * Yc
            + Yc ** 2
        )
        - R0
        * Yc
        * sqrt(
            -(R0 ** 2)
            + X5 ** 2
            - 2 * X5 * Xc
            + Xc ** 2
            + Y5 ** 2
            - 2 * Y5 * Yc
            + Yc ** 2
        )
        + X5 ** 2 * Xc
        - 2 * X5 * Xc ** 2
        + Xc ** 3
        + Xc * Y5 ** 2
        - 2 * Xc * Y5 * Yc
        + Xc * Yc ** 2
    ) / (X5 ** 2 - 2 * X5 * Xc + Xc ** 2 + Y5 ** 2 - 2 * Y5 * Yc + Yc ** 2)
    Y4 = (
        R0 ** 2 * Y5
        - R0 ** 2 * Yc
        - R0
        * (X5 - Xc)
        * sqrt(
            -(R0 ** 2)
            + X5 ** 2
            - 2 * X5 * Xc
            + Xc ** 2
            + Y5 ** 2
            - 2 * Y5 * Yc
            + Yc ** 2
        )
        + X5 ** 2 * Yc
        - 2 * X5 * Xc * Yc
        + Xc ** 2 * Yc
        + Y5 ** 2 * Yc
        - 2 * Y5 * Yc ** 2
        + Yc ** 3
    ) / (X5 ** 2 - 2 * X5 * Xc + Xc ** 2 + Y5 ** 2 - 2 * Y5 * Yc + Yc ** 2)
    Z4 = X4 + 1j * Y4

    Z9 = Z4.conjugate()
    Z10 = Z3.conjugate()
    Zc2 = Zc1.conjugate()

    surf_list = list()
    # Create all the surfaces for all the cases
    # Air surface Zc1 with magnet_0
    curve_list_air = list()
    if self.W2 > 0:
        curve_list_air.append(Segment(Z1, Z2))
    curve_list_air.append(Segment(Z2, Z3))
    curve_list_air.append(Arc1(begin=Z3, end=Z4, radius=R0, is_trigo_direction=True))
    curve_list_air.append(Segment(Z4, Z5))
    if self.W2 > 0:
        curve_list_air.append(Segment(Z5, Z6))
    curve_list_air.append(Segment(Z6, Z1))
    point_ref = Zc1
    S1 = SurfLine(line_list=curve_list_air, label="Hole" + st, point_ref=point_ref)

    # Magnet_0 surface
    curve_list_mag = list()
    curve_list_mag.append(Segment(Z1, Z6))
    curve_list_mag.append(Segment(Z6, Z7))
    curve_list_mag.append(Segment(Z7, Z12))
    curve_list_mag.append(Segment(Z12, Z1))
    point_ref = (Z1 + Z6 + Z7 + Z12) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_0:
        if self.magnet_0.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T0_S0"
    S2 = SurfLine(line_list=curve_list_mag, label=magnet_label, point_ref=point_ref)

    # Air surface Zc2 with magnet_0
    curve_list_air = list()
    curve_list_air.append(Segment(Z12, Z7))
    if self.W2 + self.W1 < self.W0:
        curve_list_air.append(Segment(Z7, Z8))
    curve_list_air.append(Segment(Z8, Z9))
    curve_list_air.append(Arc1(begin=Z9, end=Z10, radius=R0, is_trigo_direction=True))
    curve_list_air.append(Segment(Z10, Z11))
    if self.W2 + self.W1 < self.W0:
        curve_list_air.append(Segment(Z11, Z12))
    point_ref = Zc2
    S3 = SurfLine(line_list=curve_list_air, label="Hole" + st, point_ref=point_ref)

    # Full surface, no magnet
    curve_list_air = list()
    curve_list_air.append(Segment(Z2, Z3))
    curve_list_air.append(Arc1(begin=Z3, end=Z4, radius=R0, is_trigo_direction=True))
    curve_list_air.append(Segment(Z4, Z5))
    curve_list_air.append(Segment(Z5, Z8))
    curve_list_air.append(Segment(Z8, Z9))
    curve_list_air.append(Arc1(begin=Z9, end=Z10, radius=R0, is_trigo_direction=True))
    curve_list_air.append(Segment(Z10, Z11))
    curve_list_air.append(Segment(Z11, Z2))
    point_ref = Rbo - self.H0 - self.H2 / 2
    S4 = SurfLine(line_list=curve_list_air, label="Hole" + st, point_ref=point_ref)

    # Create the surface list by selecting the correct ones
    if self.magnet_0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        surf_list = [S1, S2, S3]
    else:
        S4.label = S1.label + "_R0_T0_S0"  # Hole
        surf_list = [S4]

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QDialog, QMessageBox

from .....Classes.Magnet import Magnet
from .....Classes.MagnetType13 import MagnetType13
from .....Classes.Slot import Slot
from .....GUI.Dialog.DMachineSetup.SMagnet.PMagnet10.PMagnet10 import PMagnet10
from .....GUI.Dialog.DMachineSetup.SMagnet.PMagnet11.PMagnet11 import PMagnet11
from .....GUI.Dialog.DMachineSetup.SMagnet.PMagnet12.PMagnet12 import PMagnet12
from .....GUI.Dialog.DMachineSetup.SMagnet.PMagnet13.PMagnet13 import PMagnet13
from .....GUI.Dialog.DMachineSetup.SMagnet.PMagnet14.PMagnet14 import PMagnet14
from .....GUI.Dialog.DMachineSetup.SMagnet.Ui_SMagnet import Ui_SMagnet


class SMagnet(Ui_SMagnet, QDialog):
    """Step to set the magnet (and slot) for SPMSM/SIPMSM"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineType nav
    step_name = "Magnet"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : SMagnet
            A SMagnet widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """
        # Build the interface according to the .ui file
        QDialog.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Get the correct available widgets according to the machine
        if self.machine.rotor.is_internal:
            self.wid_list = [PMagnet10, PMagnet11, PMagnet12, PMagnet13, PMagnet14]
        elif self.machine.type_machine == 6:
            # For SPMSM only polar magnet for external rotor
            self.wid_list = [PMagnet11]
        else:
            # Magnet 14 not available for SIPMSM
            self.wid_list = [PMagnet10, PMagnet11, PMagnet12, PMagnet13]
        self.mag_type_index = [wid.mag_type for wid in self.wid_list]
        self.slot_type_index = [wid.slot_type for wid in self.wid_list]
        self.mag_name_index = [wid.mag_name for wid in self.wid_list]

        # Help URL
        self.b_help.url = "https://eomys.com/produits/manatee/howtos/article/"
        self.b_help.url += "how-to-set-up-inset-magnet"

        # Fill the combobox with the available magnet
        self.c_type.clear()
        for mag in self.mag_name_index:
            self.c_type.addItem(mag)

        # Avoid erase all the parameters when navigating though the magnets
        self.previous_mag = dict()
        for mag_type in self.mag_type_index:
            self.previous_mag[mag_type] = None

        # Set the GUI with the current values if provided
        if (
            type(self.machine.rotor.slot) is Slot
            or type(self.machine.rotor.slot.magnet[0]) is Magnet
        ):
            # Magnet or slot not set
            # Type 11 is default
            self.set_type(1)
            self.c_type.setCurrentIndex(1)
        else:  # The Magnet is set => load the parameter
            # Set the type
            index = self.mag_type_index.index(type(self.machine.rotor.slot.magnet[0]))
            self.set_type_gui(index)
            self.c_type.setCurrentIndex(index)
        # Only one magnet in the GUI (for now)
        self.machine.rotor.slot.W3 = 0

        # Set magnetization
        if self.machine.rotor.slot.magnet[0].type_magnetization is None:
            self.machine.rotor.slot.magnet[0].type_magnetization = 0
        self.c_type_magnetization.setCurrentIndex(
            self.machine.rotor.slot.magnet[0].type_magnetization
        )

        self.set_slot_pitch()

        # Set material
        self.w_mat.setText(self.tr("mat_mag:"))
        self.w_mat.def_mat = "Magnet1"
        self.w_mat.update(self.machine.rotor.slot.magnet[0], "mat_type", self.matlib)

        # Connect signals
        self.c_type.currentIndexChanged.connect(self.set_type)
        self.c_type_magnetization.currentIndexChanged.connect(
            self.s_set_type_magnetization
        )
        self.b_plot.clicked.connect(self.s_plot)
        self.w_mat.saveNeeded.connect(self.emit_save)

    def set_slot_pitch(self):
        """Update the slot pitch text

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        """
        Nmag_txt = self.tr("Number of magnets = 2p = ")
        if self.machine.rotor.slot.Zs is not None:
            Zs = self.machine.rotor.slot.Zs
            out = Nmag_txt + str(Zs) + " => "
            Slot_pitch = 360.0 / Zs
            Slot_pitch_rad = Slot_pitch * pi / 180

            pitch_txt = self.tr("Slot pitch = ")
            out += (
                pitch_txt
                + "%.4g" % (Slot_pitch)
                + u" ° ("
                + "%.4g" % (Slot_pitch_rad)
                + " rad)"
            )
            self.out_Nmag.setText(out)
        else:
            self.out_Nmag.setText(Nmag_txt + "?")

    def emit_save(self):
        """Emit the saveNeeded signal for the DMachineSetup"""
        self.saveNeeded.emit()

    def set_type_gui(self, index):
        """Change the GUI to the correct Magnet widget

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        index : int
            Index of the Magnet type to use
        """
        # Regenerate the pages with the new values
        self.w_mag.setParent(None)
        self.w_mag = self.wid_list[index](self.machine)
        self.w_mag.saveNeeded.connect(self.emit_save)

        # Refresh the GUI
        self.main_layout.removeWidget(self.w_mag)
        self.main_layout.insertWidget(2, self.w_mag)

    def set_type_obj(self, index):
        """Set the type of magnet (update only the object)

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        index : int
            Index of the Magnet type to use
        """
        # Save the mag
        slot = self.machine.rotor.slot
        self.previous_mag[type(slot.magnet[0])] = slot

        if self.previous_mag[self.mag_type_index[index]] is None:
            # Set the slot
            Zs = self.machine.rotor.slot.Zs
            self.machine.rotor.slot = self.slot_type_index[index]()
            self.machine.rotor.slot._set_None()
            self.machine.rotor.slot.Zs = Zs
            self.machine.rotor.slot.W3 = 0
            # Set the magnet
            self.machine.rotor.slot.magnet = list()
            mag = self.mag_type_index[index]()
            self.machine.rotor.slot.magnet.append(mag)
            self.machine.rotor.slot.magnet[0]._set_None()
        else:
            self.machine.rotor.slot = self.previous_mag[self.mag_type_index[index]]

    def set_type(self, index):
        """Signal to set the type of magnet (update both the object and the gui)

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        index : int
            Index of the Magnet type to use
        """
        self.set_type_obj(index)
        self.set_type_gui(index)
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def s_plot(self):
        """Plot the current machine

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        """
        # We have to make sure the slot is right before truing to plot it
        error = self.check(self.machine.rotor)

        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Plot the machine
            self.machine.plot()

    @staticmethod
    def check(lamination):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lamination : Lamination
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """
        try:
            # Check that everything is set
            if lamination.slot.magnet[0].Wmag is None:
                return "You must set Wmag !"
            if lamination.slot.magnet[0].Hmag is None:
                return "You must set Hmag !"
            if (
                hasattr(lamination.slot.magnet[0], "Rtop")
                and lamination.slot.magnet[0].Rtop is None
            ):
                return "You must set Rtopm !"
            if lamination.slot.H0 is None:
                return "You must set H0 !"
        except Exception as e:
            return str(e)

        # Check that everything is set right
        try:
            mec_gap = lamination.parent.comp_width_airgap_mec()
        except:
            return "Unable to draw the magnet, " "please check your geometry !"

        if mec_gap <= 0:
            return "You must have gap_min > 0 (reduce Hmag) !"
        if hasattr(lamination.slot.magnet[0], "Rtop"):
            if (
                type(lamination.slot.magnet[0]) is MagnetType13
                and lamination.slot.magnet[0].Rtop
                < lamination.slot.magnet[0].Wmag / 2.0
            ):
                return "You must have Rtopm >= Wmag/2 !"

    def s_set_type_magnetization(self, index):
        """Signal to update the value of type_magnetization according to the combobox

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        index : int
            Current index of the combobox
        """
        self.machine.rotor.slot.magnet[0].type_magnetization = index
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()



# -*- coding: utf-8 -*-
from numpy import pi
from SciDataTool import Data1D, DataLinspace, DataTime
from ....Functions.Winding.gen_phase_list import gen_name
from ....Functions.Plot.plot_A_space import plot_A_space
from ....definitions import config_dict


def plot_mmf_unit(self, fig=None):
    """Plot the winding unit mmf as a function of space
    Parameters
    ----------
    self : LamSlotWind
        an LamSlotWind object
    Na : int
        Space discretization
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    """

    # Compute the winding function and mmf
    wf = self.comp_wind_function()
    qs = self.winding.qs
    mmf_u = self.comp_mmf_unit(Nt=1, Na=wf.shape[1])

    if len(mmf_u.values.shape) == 1:
        mmf_u.values = mmf_u.values[None, :]  # TODO: correct bug in SciDataTool

    # Create a Data object
    Phase = Data1D(
        name="phase",
        unit="",
        values=gen_name(qs, is_add_phase=True),
        is_components=True,
    )
    Angle = mmf_u.axes[1]
    WF = DataTime(
        name="WF",
        unit="p.u.",
        symbol="Magnitude",
        axes=[Phase, Angle],
        values=wf,
        symmetries=mmf_u.symmetries,
    )

    color_list = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][: qs + 1]
    plot_A_space(
        WF,
        is_fft=True,
        index_list=[0, 1, 2],
        data_list=[mmf_u],
        fig=fig,
        color_list=color_list,
    )



# -*- coding: utf-8 -*-
import numpy as np


def get_axis(self):
    """Get the value of variables stored in Solution.

    Parameters
    ----------
    self : SolutionMat
        an SolutionMat object

    Returns
    -------
    axis_dict: dict
        a dict of axis names containing axis sizes

    """

    return self.axis



# -*- coding: utf-8 -*-

from ..init_fig import init_fig
from .plot_A_3D import plot_A_3D
from numpy import max as np_max


def plot_A_cfft2(
    data,
    is_elecorder=False,
    is_spaceorder=False,
    freq_max=20000,
    r_max=100,
    mag_max=None,
    N_stem=100,
    disp_negative=False,
    is_norm=False,
    unit="SI",
    save_path=None,
    fig=None,
    subplot_index=None,
):
    """3D stem plot of the 2D Fourier Transform of a field

    Parameters
    ----------
    data : Data
        a Data object
    is_elecorder : bool
        boolean indicating if we want to use the electrical order for the fft axis
    is_spaceorder : bool
        boolean indicating if we want to use the spatial order for the fft axis
    freq_max : int
        maximum value of the frequency for the fft axis
    r_max : int
        maximum value of the wavenumber for the fft axis
    mag_max : int
        maximum value of the magnitude
    N_stem : int
        number of stems to plot
    disp_negative : bool
        plot negative frequencies
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    save_path : str
        path and name of the png file to save
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    """

    # Set plot
    is_show_fig = True if fig is None else False
    (fig, axes, patch_leg, label_leg) = init_fig(fig, shape="rectangle")
    title = "Complex FFT2 of " + data.name
    if is_elecorder:
        xlabel = "Electrical order []"
        freq_max = freq_max / data.normalizations.get("elec_order")
        if disp_negative:
            x_str = "freqs=[-" + str(freq_max) + "," + str(freq_max) + "]{elec_order}"
            x_min = -freq_max
        else:
            x_str = "freqs=[0," + str(freq_max) + "]{elec_order}"
            x_min = 0
    else:
        xlabel = "Frequency [Hz]"
        if disp_negative:
            x_str = "freqs=[-" + str(freq_max) + "," + str(freq_max) + "]"
            x_min = -freq_max
        else:
            x_str = "freqs=[0," + str(freq_max) + "]"
            x_min = 0
    if is_spaceorder:
        ylabel = "Spatial order []"
        r_max = r_max / data.normalizations.get("space_order")
        y_str = "wavenumber=[-" + str(r_max) + "," + str(r_max) + "]{space_order}"
    else:
        ylabel = "Wavenumber []"
        y_str = "wavenumber=[-" + str(r_max) + "," + str(r_max) + "]"
    if unit == "SI":
        unit = data.unit
        unit_str = "[" + unit + "]"
    elif "dB" in unit:
        unit_str = (
            "[" + unit + " re. " + str(data.normalizations["ref"]) + data.unit + "]"
        )
    else:
        unit_str = "[" + unit + "]"

    if data.symbol == "Magnitude":
        zlabel = "Magnitude " + unit_str
    else:
        zlabel = r"$|\widehat{" + data.symbol + "}|$ " + unit_str

    # Extract the field
    results = data.get_harmonics(
        N_stem, x_str, y_str, unit=unit, is_norm=False, is_flat=True
    )
    F_flat = results["freqs"]
    R_flat = results["wavenumber"]
    B_FT_flat = results[data.symbol]
    if mag_max is None:
        mag_max = np_max(B_FT_flat)

    # Plot the original graph
    plot_A_3D(
        F_flat,
        R_flat,
        B_FT_flat,
        fig=fig,
        x_min=x_min,
        x_max=freq_max,
        y_min=r_max,
        y_max=-r_max,
        z_min=0,
        z_max=mag_max,
        title=title,
        xlabel=xlabel,
        ylabel=ylabel,
        zlabel=zlabel,
        type="stem",
        save_path=save_path,
        subplot_index=subplot_index,
    )

    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-
"""File generated according to PMagnet12/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SMagnet.PMagnet12.Ui_PMagnet12 import (
    Ui_PMagnet12,
)


class Gen_PMagnet12(Ui_PMagnet12):
    def setupUi(self, PMagnet12):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PMagnet12.setupUi(self, PMagnet12)
        # Setup of in_Hmag
        txt = self.tr(u"""magnet radial height [m]""")
        self.in_Hmag.setWhatsThis(txt)
        self.in_Hmag.setToolTip(txt)

        # Setup of lf_Hmag
        self.lf_Hmag.validator().setBottom(0)
        txt = self.tr(u"""magnet radial height [m]""")
        self.lf_Hmag.setWhatsThis(txt)
        self.lf_Hmag.setToolTip(txt)

        # Setup of in_Wmag
        txt = self.tr(u"""magnet bottom width [m]""")
        self.in_Wmag.setWhatsThis(txt)
        self.in_Wmag.setToolTip(txt)

        # Setup of lf_Wmag
        self.lf_Wmag.validator().setBottom(0)
        txt = self.tr(u"""magnet bottom width [m]""")
        self.lf_Wmag.setWhatsThis(txt)
        self.lf_Wmag.setToolTip(txt)



# -*- coding: utf-8 -*-
from numpy import pi


def comp_surface(self):
    """Compute the Magnet surface (by numerical computation)

    Parameters
    ----------
    self : MagnetType11
        A MagnetType11 object

    Returns
    -------
    S: float
        Magnet surface [m**2]

    """

    [Z1, Z2, Z3, Zs3, Zs4, Z4, Zref] = self._comp_point_coordinate()
    Rslot = abs(Z1)
    Rtop = abs(Z3)

    if self.is_outwards():
        S = pi * (Rslot ** 2 - Rtop ** 2)
    else:
        S = pi * (Rtop ** 2 - Rslot ** 2)
    return S * self.Wmag / (2 * pi)



# -*- coding: utf-8 -*-

from numpy import arcsin


def comp_angle_opening_magnet(self):
    """Compute the opening angle at the top of the slot

    Parameters
    ----------
    self : SlotMFlat2
        A SlotMFlat2 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    if self.W1 > 0:
        Rbo = self.get_Rbo()
        return float(2 * arcsin(self.W1 / (2 * Rbo)))
    else:
        return self.comp_angle_magnet()



# -*- coding: utf-8 -*-

from numpy import cos, tan


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW11
        A SlotW11 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    Rbo = self.get_Rbo()

    if self.H1_is_rad:  # H1 in rad
        H1 = (self.W1 - self.W0) / 2.0 * tan(self.H1)  # convertion to m
    else:  # H1 in m
        H1 = self.H1

    # Computation of the arc height
    alpha = self.comp_angle_opening() / 2
    Harc = float(Rbo * (1 - cos(alpha)))

    if self.is_outwards():
        return (
            abs(Rbo - Harc + self.H0 + H1 + self.H2 + 1j * (self.W2 / 2.0 - self.R1))
            - Rbo
        )
    else:
        return self.H0 + H1 + self.H2 + Harc



# -*- coding: utf-8 -*-

from numpy import arcsin, pi


def comp_alphas(self):
    """Compute the opening angle to have a constant slot.

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    (alpha_0,alpha_2): tuple
        Angle at top and bottom of the slot [rad]

    """
    Rbo = self.get_Rbo()

    # alpha_Tt is the angle of the tooth to have the correct top width
    alpha_Tt = 2 * float(arcsin(self.W3 / (2 * Rbo)))

    # alpha_0 + alpha_Tt = slot_ptich
    # Zs * (alpha_0+alpha_Tt) = 2 pi
    alpha_0 = 2 * pi / self.Zs - alpha_Tt

    if self.is_outwards():
        # alpha_Tb is the angle of the tooth to have the correct bottom width
        alpha_Tb = 2 * float(arcsin(self.W3 / (2 * (Rbo + self.H2))))
    else:
        alpha_Tb = 2 * float(arcsin(self.W3 / (2 * (Rbo - self.H2))))

    # Zs * (alpha_2+alpha_Tb) = 2 pi
    alpha_2 = 2 * pi / self.Zs - alpha_Tb

    return (alpha_0, alpha_2)



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch
from matplotlib.pyplot import axis, legend

from ....Functions.Winding.find_wind_phase_color import find_wind_phase_color
from ....Functions.Winding.gen_phase_list import gen_name
from ....Functions.init_fig import init_fig
from ....definitions import config_dict
from ....Classes.WindingSC import WindingSC

PHASE_COLORS = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"]
ROTOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["ROTOR_COLOR"]
STATOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["STATOR_COLOR"]


def plot(
    self,
    fig=None,
    is_lam_only=False,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_display=True,
    is_show=True,
):
    """Plot the Lamination in a matplotlib fig

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object
    fig :
        if None, open a new fig and plot, else add to the current
        one (Default value = None)
    is_lam_only : bool
        True to plot only the lamination (remove the Winding)
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    is_display : bool
        False to return the patches
    is_show : bool
        To call show at the end of the method
    Returns
    -------
    patches : list
        List of Patches
    """
    if self.is_stator:
        color_lam = STATOR_COLOR
    else:
        color_lam = ROTOR_COLOR

    # Get the LamSlot surface(s)
    surf_list = self.build_geometry(sym=sym, alpha=alpha, delta=delta)

    patches = list()
    # getting the matrix  wind_mat [Nrad,Ntan,Zs,qs] representing the winding
    if type(self.winding) is WindingSC:
        wind_mat = None
    else:
        try:
            Zs = self.get_Zs()
            wind_mat = self.winding.comp_connection_mat(Zs)
            (Nrad, Ntan, Zs, qs) = wind_mat.shape
        except Exception:
            wind_mat = None
    if wind_mat is None:
        qs = 1  # getting number of surface in winding Zone in the Slot
    for surf in surf_list:
        if surf.label is not None and "Lamination" in surf.label:
            patches.extend(surf.get_patches(color_lam, is_edge_only=is_edge_only))
        elif "Wind" in surf.label or "Bar" in surf.label:
            if not is_lam_only:
                color = find_wind_phase_color(wind_mat=wind_mat, label=surf.label)
                patches.extend(surf.get_patches(color=color, is_edge_only=is_edge_only))
        else:
            patches.extend(surf.get_patches(is_edge_only=is_edge_only))

    if is_display:
        # Display the result
        (fig, axes, patch_leg, label_leg) = init_fig(fig)
        axes.set_xlabel("(m)")
        axes.set_ylabel("(m)")
        for patch in patches:
            axes.add_patch(patch)
        # Axis Setup
        axis("equal")

        # The Lamination is centered in the figure
        Lim = self.Rext * 1.5
        axes.set_xlim(-Lim, Lim)
        axes.set_ylim(-Lim, Lim)

        # Add the legend
        if not is_edge_only:
            if self.is_stator and "Stator" not in label_leg:
                patch_leg.append(Patch(color=STATOR_COLOR))
                label_leg.append("Stator")
                axes.set_title("Stator with Winding")
            elif not self.is_stator and "Rotor" not in label_leg:
                patch_leg.append(Patch(color=ROTOR_COLOR))
                label_leg.append("Rotor")
                axes.set_title("Rotor with Winding")
            # Add the winding legend only if needed
            if not is_lam_only:
                phase_name = gen_name(qs, is_add_phase=True)
                for ii in range(qs):
                    if not phase_name[ii] in label_leg:
                        # Avoid adding twice the same label
                        index = ii % len(PHASE_COLORS)
                        patch_leg.append(Patch(color=PHASE_COLORS[index]))
                        label_leg.append(phase_name[ii])
            legend(patch_leg, label_leg)
        if is_show:
            fig.show()
    else:
        return patches



# -*- coding: utf-8 -*-
from numpy import cos


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW15
        A SlotW15 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    return self.H0 + self.H1 + self.H2



# -*- coding: utf-8 -*-


def comp_surface_wind(self, Ndisc=200):
    """Compute the Slot winding surface (by numerical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotWind
        A SlotWind object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    surf = self.build_geometry_wind(Nrad=1, Ntan=1)

    return surf[0].comp_surface(Ndisc=Ndisc)
